@@ -21,7 +21,7 @@
 #define _tds_h_
 
 static char rcsid_tds_h[]=
-	 "$Id: tds.h,v 1.79 2001/07/02 00:57:40 camber Exp $";
+	 "$Id: tds.h.in,v 1.39 2002/08/30 20:23:00 freddy77 Exp $";
 static void *no_unused_tds_h_warn[]={rcsid_tds_h, no_unused_tds_h_warn};
 
 #include "tds_configs.h"
@@ -51,8 +51,7 @@ static void *no_unused_tds_h_warn[]={rcsid_tds_h, no_unused_tds_h_warn};
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
-/* FIX ME -- use autoconf for the existance of malloc.h */
-#ifndef __APPLE__
+#ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif
 #include <sys/time.h>
@@ -76,45 +75,58 @@ extern "C" {
 ** missed a bunch, but the idea is we can do an ifdef here 
 ** to set the appropriately sized native type.
 **
-** If you have problems on 64bit machines and the code is 
+** If you have problems on 64-bit machines and the code is 
 ** using a native datatype, please change the code to use
 ** these. (In the TDS layer only, the API layers have their
 ** own typedefs which equate to these).
 */
-typedef char		TDS_CHAR;      /*  8 bit char     */
-typedef unsigned char	TDS_UCHAR;     /*  8 bit uchar    */
-typedef unsigned char	TDS_TINYINT;   /*  8 bit int      */
-typedef short             TDS_SMALLINT;  /* 16 bit int      */
-typedef unsigned short  TDS_USMALLINT; /* 16 bit unsigned */
-typedef int                  TDS_INT;       /* 32 bit int      */
-typedef unsigned int       TDS_UINT;      /* 32 bit unsigned */
-typedef float                 TDS_REAL;      /* 32 bit float    */
-typedef double                TDS_FLOAT;     /* 64 bit float    */
+typedef char				TDS_CHAR;      /*  8 bit char     */
+typedef unsigned char		TDS_UCHAR;     /*  8 bit uchar    */
+typedef unsigned char		TDS_TINYINT;   /*  8 bit int      */
+typedef short            TDS_SMALLINT;  /* 16 bit int      */
+typedef unsigned short  	TDS_USMALLINT; /* 16 bit unsigned */
+typedef int                 TDS_INT;       /* 32 bit int      */
+typedef unsigned int       	TDS_UINT;      /* 32 bit unsigned */
+typedef float                TDS_REAL;      /* 32 bit float    */
+typedef double               TDS_FLOAT;     /* 64 bit float    */
+typedef long long			TDS_INT8;     	/* 64 bit integer  */
+typedef unsigned long long	TDS_UINT8;     	/* 64 bit integer  */
+
 typedef struct tdsnumeric
 {
         unsigned char         precision;
         unsigned char         scale;
-        unsigned char         array[33];
+        unsigned char         array[17];
 } TDS_NUMERIC;
-typedef struct tdsmoney
+
+typedef struct tdsoldmoney
 {
-        TDS_INT         mnyhigh;
-        TDS_INT         mnylow;
+	TDS_INT         mnyhigh;
+	TDS_INT         mnylow;
+} TDS_OLD_MONEY;
+
+typedef union tdsmoney
+{	TDS_OLD_MONEY	tdsoldmoney;
+        TDS_INT8        mny;
 } TDS_MONEY;
+
 typedef struct tdsmoney4
-{
+{	
         TDS_INT          mny4;
 } TDS_MONEY4;
+
 typedef struct tdsdatetime
 {
 	TDS_INT dtdays;
 	TDS_INT dttime;
 } TDS_DATETIME;
+
 typedef struct tdsdatetime4
 {
 	TDS_USMALLINT days;
 	TDS_USMALLINT minutes;
 } TDS_DATETIME4;
+
 typedef struct tdsvarbinary
 {
 	TDS_INT len;
@@ -126,6 +138,28 @@ typedef struct tdsvarchar
 	TDS_CHAR array[256];
 } TDS_VARCHAR;
 
+typedef struct tdsunique
+{
+    TDS_UINT      Data1;
+    TDS_USMALLINT Data2;
+    TDS_USMALLINT Data3;
+    TDS_UCHAR     Data4[8];
+} TDS_UNIQUE;
+
+typedef struct tdsdaterec
+{
+	TDS_INT   year;
+	TDS_INT   month;
+	TDS_INT   day;
+	TDS_INT   dayofyear;
+	TDS_INT   weekday;
+	TDS_INT   hour;
+	TDS_INT   minute;
+	TDS_INT   second;
+	TDS_INT   millisecond;
+	TDS_INT   tzone;
+} TDSDATEREC;
+
 #define TDS_NO_MORE_ROWS     -2
 #define TDS_SUCCEED          1
 #define TDS_FAIL             0
@@ -138,6 +172,9 @@ typedef struct tdsvarchar
 #define TDS_ERROR            3  
 #define TDS_DONT_RETURN      42
 
+#define TDS5_DYN_TOKEN      231  /* 0xE7    TDS 5.0 only              */
+#define TDS5_DYNRES_TOKEN   236  /* 0xEC    TDS 5.0 only              */
+#define TDS5_DYN3_TOKEN     215  /* 0xD7    TDS 5.0 only              */
 #define TDS_LANG_TOKEN       33  /* 0x21    TDS 5.0 only              */
 #define TDS_CLOSE_TOKEN     113  /* 0x71    TDS 5.0 only? ct_close()  */
 #define TDS_RET_STAT_TOKEN  121  /* 0x79                              */
@@ -169,7 +206,7 @@ typedef struct tdsvarchar
 /* states for tds_process_messages() */
 #define PROCESS_ROWS    0
 #define PROCESS_RESULTS 1
-#define CANCEL          2
+#define CANCEL_PROCESS  2
 #define GOTO_1ST_ROW    3
 #define LOGIN           4
 
@@ -189,6 +226,7 @@ sheesh! </rant>
 #define SYBINT1      48   /* 0x30 */
 #define SYBINT2      52   /* 0x34 */
 #define SYBINT4      56   /* 0x38 */
+#define SYBINT8     127   /* 0x7F */
 #define SYBFLT8      62   /* 0x3E */
 #define SYBDATETIME  61   /* 0x3D */
 #define SYBBIT       50   /* 0x32 */
@@ -209,13 +247,28 @@ sheesh! </rant>
 #define SYBFLTN      109  /* 0x6D */
 #define SYBMONEYN    110  /* 0x6E */
 #define SYBDATETIMN  111  /* 0x6F */
-#define XSYBCHAR     167  /* 0xA7 */
-#define XSYBVARCHAR  175  /* 0xAF */
+#define XSYBCHAR     175  /* 0xAF */
+#define XSYBVARCHAR  167  /* 0xA7 */
 #define XSYBNVARCHAR 231  /* 0xE7 */
 #define XSYBNCHAR    239  /* 0xEF */
+#define XSYBVARBINARY 165  /* 0xA5 */
+#define XSYBBINARY    173  /* 0xAD */
+
+#define SYBUNIQUE    36    /* 0x24 */
+#define SYBVARIANT   0x62
 
 #define TDS_ZERO_FREE(x) {free((x)); (x) = NULL;}
 
+#define TDS_BYTE_SWAP16(value)                 \
+         (((((unsigned short)value)<<8) & 0xFF00)   | \
+          ((((unsigned short)value)>>8) & 0x00FF))
+
+#define TDS_BYTE_SWAP32(value)                     \
+         (((((unsigned long)value)<<24) & 0xFF000000)  | \
+          ((((unsigned long)value)<< 8) & 0x00FF0000)  | \
+          ((((unsigned long)value)>> 8) & 0x0000FF00)  | \
+          ((((unsigned long)value)>>24) & 0x000000FF))
+
 #define is_end_token(x) (x==TDS_DONE_TOKEN    || \
 			x==TDS_DONEPROC_TOKEN    || \
 			x==TDS_DONEINPROC_TOKEN)
@@ -236,13 +289,15 @@ sheesh! </rant>
 #define is_fixed_type(x) (x==SYBINT1    || \
 			x==SYBINT2      || \
 			x==SYBINT4      || \
-			x==SYBREAL	|| \
+			x==SYBINT8      || \
+			x==SYBREAL	 || \
 			x==SYBFLT8      || \
 			x==SYBDATETIME  || \
 			x==SYBDATETIME4 || \
 			x==SYBBIT       || \
 			x==SYBMONEY     || \
-			x==SYBMONEY4)
+			x==SYBMONEY4    || \
+			x==SYBUNIQUE)
 #define is_nullable_type(x) (x==SYBINTN || \
 			x==SYBFLTN      || \
 			x==SYBDATETIMN  || \
@@ -257,11 +312,13 @@ sheesh! </rant>
 /* large type means it has a two byte size field */
 #define is_large_type(x) (x>128)
 #define is_numeric_type(x) (x==SYBNUMERIC || x==SYBDECIMAL)
-#define is_unicode(x) (x==XSYBNVARCHAR || x==XSYBNCHAR)
+#define is_unicode(x) (x==XSYBNVARCHAR || x==XSYBNCHAR || x==SYBNTEXT)
+#define is_collate_type(x) (x==XSYBVARCHAR || x==XSYBCHAR || x==SYBTEXT || x == XSYBNVARCHAR || x==SYBNTEXT)
 
 #define TDS_MAX_CAPABILITY	18
 #define MAXPRECISION 		50
 #define TDS_MAX_CONN		4096
+#define TDS_MAX_DYNID_LEN	30
 
 /* defaults to use if no others are found */
 #define TDS_DEF_SERVER		"SYBASE"
@@ -280,6 +337,10 @@ sheesh! </rant>
 #define TDS_DEF_MAJOR		7
 #define TDS_DEF_MINOR		0
 #define TDS_DEF_PORT		1433
+#elif TDS80
+#define TDS_DEF_MAJOR		8
+#define TDS_DEF_MINOR		0
+#define TDS_DEF_PORT		1433
 #else
 #define TDS_DEF_MAJOR		5
 #define TDS_DEF_MINOR		0
@@ -302,21 +363,32 @@ sheesh! </rant>
 #define TDS_STR_HOSTNAME "hostname"
 #define TDS_STR_HOST     "host"
 #define TDS_STR_PORT     "port"
+#define TDS_STR_TEXTSZ   "text size"
+/* for big endian hosts */
+#define TDS_STR_EMUL_LE	"emulate little endian"
+#define TDS_STR_CHARSET	"charset"
+#define TDS_STR_CLCHARSET	"client charset"
+#define TDS_STR_LANGUAGE	"language"
+#define TDS_STR_APPENDMODE	"dump file append"
+#define TDS_STR_DATEFMT	"date format"
 
-
+#define TDS_MAX_LOGIN_STR_SZ 30
+#define TDS_MAX_LIBRARY_STR_SZ 11
 typedef struct tds_login {
-	TDS_CHAR  host_name[30];
-	TDS_CHAR  user_name[30];
-	TDS_CHAR  password[30];
+	TDS_CHAR  host_name[TDS_MAX_LOGIN_STR_SZ+1];
+	TDS_CHAR  user_name[TDS_MAX_LOGIN_STR_SZ+1];
+	/* FIXME temporary fix, 40 pwd max len */
+	TDS_CHAR  password[40+1];
 	TDS_TINYINT bulk_copy; 
-	TDS_CHAR  app_name[30];
-	TDS_CHAR  server_name[30];
+	TDS_CHAR  app_name[TDS_MAX_LOGIN_STR_SZ+1];
+	TDS_CHAR  server_name[TDS_MAX_LOGIN_STR_SZ+1];
 	TDS_TINYINT  major_version; /* TDS version */
 	TDS_TINYINT  minor_version; /* TDS version */
-	TDS_CHAR  library[11]; /* Ct-Library or DB-Library */
-	TDS_CHAR language[30]; /* ie us-english */
+	/* Ct-Library, DB-Library,  TDS-Library or ODBC */
+	TDS_CHAR  library[TDS_MAX_LIBRARY_STR_SZ+1];
+	TDS_CHAR language[TDS_MAX_LOGIN_STR_SZ+1]; /* ie us-english */
 	TDS_TINYINT encrypted; 
-	TDS_CHAR char_set[30]; /*  ie iso_1 */
+	TDS_CHAR char_set[TDS_MAX_LOGIN_STR_SZ+1]; /*  ie iso_1 */
 	TDS_SMALLINT block_size; 
 	TDS_TINYINT suppress_language;
      TDS_INT connect_timeout;
@@ -350,15 +422,34 @@ typedef struct tds_config_info {
         int try_domain_login;
         int xdomain_auth;
         int debug_level;
+        int emul_little_endian;
+        int text_size;
+        char *app_name;
+        char *user_name;
+        char *password;
+        char *library;
+        int bulk_copy;
+	   int suppress_language;
+	   int encrypted;
+        char *client_charset;
 } TDSCONFIGINFO;
 
+typedef struct tds_loc_info {
+        char *language;
+        char *char_set;
+        char *date_fmt;
+} TDSLOCINFO;
+
 /* structure for storing data about regular and compute rows */ 
 typedef struct tds_column_info {
 	TDS_SMALLINT column_type;
-	TDS_SMALLINT column_xtype;
+	TDS_SMALLINT column_type_save;
 	TDS_SMALLINT column_usertype;
+	TDS_SMALLINT column_flags;
 	TDS_INT column_size;
 	TDS_INT column_offset;
+	TDS_TINYINT column_namelen;
+	TDS_TINYINT column_varint_size;
 	TDS_CHAR column_name[256];
 	TDS_SMALLINT column_bindtype;
 	TDS_SMALLINT column_bindfmt;
@@ -370,9 +461,16 @@ typedef struct tds_column_info {
 	TDS_SMALLINT column_scale;
 	TDS_INT column_textsize;
 	TDS_INT column_textpos;
+	TDS_INT column_text_sqlgetdatapos;
 	TDS_CHAR column_textptr[16];
 	TDS_CHAR column_timestamp[8];
 	TDS_CHAR *column_textvalue;
+	TDS_TINYINT column_nullable;
+	TDS_TINYINT column_writeable;
+	TDS_TINYINT column_identity;
+	TDS_TINYINT column_unicodedata;
+    TDS_CHAR    collation[5];
+	TDS_INT cur_row_size; /* size of this column in the current row */
 } TDSCOLINFO;
 
 typedef struct tds_result_info {
@@ -419,10 +517,17 @@ typedef struct tds_param_info {
 	unsigned char *current_row;
 } TDSPARAMINFO;
 
+typedef struct tds_input_param {
+	TDS_SMALLINT column_type;
+	TDS_CHAR *varaddr;
+	TDS_UINT column_bindlen;
+	TDS_CHAR is_null;
+} TDSINPUTPARAM;
+
 typedef struct tds_msg_info {
       TDS_SMALLINT priv_msg_type;
       TDS_SMALLINT line_number;
-      TDS_SMALLINT msg_number;
+      TDS_UINT msg_number;
       TDS_SMALLINT msg_state;
       TDS_SMALLINT msg_level;
       TDS_CHAR *server;
@@ -441,7 +546,27 @@ typedef struct tds_env_info {
 	char *database;
 } TDSENVINFO;
 
-typedef struct tds_socket {
+typedef struct tds_dynamic {
+	char id[30];
+	int dyn_state;
+	TDSRESULTINFO *res_info;
+	int num_params;
+	TDSINPUTPARAM **params;
+} TDSDYNAMIC;
+
+/* forward declaration */
+typedef struct tds_context TDSCONTEXT;
+typedef struct tds_socket  TDSSOCKET;
+
+struct tds_context {
+	TDSLOCINFO *locale;
+	void *parent;
+	/* handler */
+	int (*msg_handler)(TDSCONTEXT*, TDSSOCKET*, TDSMSGINFO*);
+	int (*err_handler)(TDSCONTEXT*, TDSSOCKET*, TDSMSGINFO*);
+};
+
+struct tds_socket {
 	/* fixed and connect time */
         int s;
 	TDS_SMALLINT major_version;
@@ -475,82 +600,118 @@ typedef struct tds_socket {
 	long longquery_param;
 	time_t queryStarttime;
 	TDSENVINFO *env;
-} TDSSOCKET;
+	/* dynamic placeholder stuff */
+	int num_dyns;
+	int cur_dyn_elem;
+	TDSDYNAMIC **dyns;
+	int emul_little_endian;
+	char *date_fmt;
+	TDSCONTEXT *tds_ctx;
+	void *iconv_info;
 
-typedef struct tds_context {
-	TDSSOCKET *connection_list[TDS_MAX_CONN];
-} TDSCONTEXT;
+	/** config for login stuff. After login this field is NULL */
+	TDSCONFIGINFO *config;
+};
 
-void tds_set_longquery_handler(TDSLOGIN * tds_login, void * longquery_func, long longquery_param);
+void tds_set_longquery_handler(TDSLOGIN * tds_login, void (* longquery_func)(long), long longquery_param);
 void tds_set_timeouts(TDSLOGIN *tds_login, int connect, int query, int longquery);
-extern int tds_write_packet(TDSSOCKET *tds,unsigned char final);
-extern int tds_init_write_buf(TDSSOCKET *tds);
-extern void tds_free_result_info(TDSRESULTINFO *info);
-extern void tds_free_socket(TDSSOCKET *tds);
-extern void tds_free_config(TDSCONFIGINFO *config);
-extern void tds_free_all_results(TDSSOCKET *tds);
-extern void tds_free_results(TDSRESULTINFO *res_info);
-extern void tds_free_param_results(TDSPARAMINFO *param_info);
-extern void tds_free_column(TDSCOLINFO *column);
-extern void tds_free_msg(TDSMSGINFO *msg_info);
-extern int tds_put_n(TDSSOCKET *tds, unsigned char *buf, int n);
-extern int tds_put_string(TDSSOCKET *tds, unsigned char *buf, int n);
-extern int tds_put_int(TDSSOCKET *tds, TDS_INT i);
-extern int tds_put_smallint(TDSSOCKET *tds, TDS_SMALLINT si);
-extern int tds_put_tinyint(TDSSOCKET *tds, TDS_TINYINT ti);
-extern int tds_put_byte(TDSSOCKET *tds, unsigned char c);
-extern unsigned char tds_get_byte(TDSSOCKET *tds);
-extern void tds_unget_byte(TDSSOCKET *tds);
-extern TDS_INT tds_get_int(TDSSOCKET *tds);
-extern TDS_SMALLINT tds_get_smallint(TDSSOCKET *tds);
-extern char *tds_get_n(TDSSOCKET *tds, void *dest, int n);
-extern char *tds_get_string(TDSSOCKET *tds, void *dest, int n);
-extern TDSRESULTINFO *tds_alloc_results(int num_cols);
-extern TDSCOMPUTEINFO *tds_alloc_compute_results(int num_cols);
-extern TDSSOCKET *tds_alloc_socket(int bufsize);
-extern TDSCONFIGINFO *tds_get_config(TDSSOCKET *tds, TDSLOGIN *login);
-extern void *tds_alloc_row(TDSRESULTINFO *res_info);
-extern char *tds_msg_get_proc_name(TDSSOCKET *tds);
-extern TDSLOGIN *tds_alloc_login();
-extern TDSCONFIGINFO *tds_alloc_config();
-extern TDSSOCKET *tds_connect(TDSLOGIN *login);
-extern void tds_set_packet(TDSLOGIN *tds_login, short packet_size);
-extern void tds_set_port(TDSLOGIN *tds_login, int port);
-extern void tds_set_passwd(TDSLOGIN *tds_login, char *password);
-extern void tds_set_bulk(TDSLOGIN *tds_login, TDS_TINYINT enabled);
-extern void tds_set_user(TDSLOGIN *tds_login, char *username);
-extern void tds_set_app(TDSLOGIN *tds_login, char *application);
-extern void tds_set_host(TDSLOGIN *tds_login, char *hostname);
-extern void tds_set_library(TDSLOGIN *tds_login, char *library);
-extern void tds_set_server(TDSLOGIN *tds_login, char *server);
-extern void tds_set_charset(TDSLOGIN *tds_login, char *charset);
-extern void tds_set_language(TDSLOGIN *tds_login, char *language);
-extern void tds_set_version(TDSLOGIN *tds_login, short major_ver, short minor_ver);
-extern void tds_set_capabilities(TDSLOGIN *tds_login, unsigned char *capabilities, int size);
-extern int tds_submit_query(TDSSOCKET *tds, char *query);
-extern int tds_process_result_tokens(TDSSOCKET *tds);
-extern int tds_process_row_tokens(TDSSOCKET *tds);
-extern int tds_process_env_chg(TDSSOCKET *tds);
-extern int tds_process_default_tokens(TDSSOCKET *tds, int marker);
-extern TDS_INT tds_process_end(TDSSOCKET *tds, int marker, int *more, int *canceled);
-extern int tds_client_msg(TDSSOCKET *tds, int msgnum, int level, int state, int line, char *message);
-extern int tds_reset_msg_info(TDSSOCKET *tds);
-extern void  tds_set_parent(TDSSOCKET* tds, void* the_parent);
-extern void* tds_get_parent(TDSSOCKET* tds);
-extern void tds_set_null(unsigned char *current_row, int column);
-extern void tds_clr_null(unsigned char *current_row, int column);
-extern int tds_get_null(unsigned char *current_row, int column);
-extern int tds7_send_login(TDSSOCKET *tds, TDSLOGIN *login);
-extern char *tds7_ascii2unicode(const char *in_string, char *out_string, int maxlen);
-extern char *tds7_unicode2ascii(const char *in_string, char *out_string, int len);
-extern char *tds7_crypt_pass(const unsigned char *clear_pass, int len, unsigned char *crypt_pass);
+int tds_write_packet(TDSSOCKET *tds,unsigned char final);
+int tds_init_write_buf(TDSSOCKET *tds);
+void tds_free_result_info(TDSRESULTINFO *info);
+void tds_free_socket(TDSSOCKET *tds);
+void tds_free_config(TDSCONFIGINFO *config);
+void tds_free_all_results(TDSSOCKET *tds);
+void tds_free_results(TDSRESULTINFO *res_info);
+void tds_free_param_results(TDSPARAMINFO *param_info);
+void tds_free_column(TDSCOLINFO *column);
+void tds_free_msg(TDSMSGINFO *msg_info);
+int tds_put_n(TDSSOCKET *tds, const unsigned char *buf, int n);
+int tds_put_string(TDSSOCKET *tds, const char *buf, int n);
+int tds_put_int(TDSSOCKET *tds, TDS_INT i);
+int tds_put_smallint(TDSSOCKET *tds, TDS_SMALLINT si);
+int tds_put_tinyint(TDSSOCKET *tds, TDS_TINYINT ti);
+int tds_put_byte(TDSSOCKET *tds, unsigned char c);
+unsigned char tds_get_byte(TDSSOCKET *tds);
+void tds_unget_byte(TDSSOCKET *tds);
+TDS_INT tds_get_int(TDSSOCKET *tds);
+TDS_SMALLINT tds_get_smallint(TDSSOCKET *tds);
+char *tds_get_n(TDSSOCKET *tds, void *dest, int n);
+char *tds_get_string(TDSSOCKET *tds, void *dest, int n);
+char *tds_get_ntstring(TDSSOCKET *tds, char *dest, int n);
+TDSRESULTINFO *tds_alloc_results(int num_cols);
+TDSCOMPUTEINFO *tds_alloc_compute_results(int num_cols);
+TDSCONTEXT *tds_alloc_context();
+void tds_free_context(TDSCONTEXT *locale);
+TDSSOCKET *tds_alloc_socket(TDSCONTEXT *context, int bufsize);
+TDSCONFIGINFO *tds_get_config(TDSSOCKET *tds, TDSLOGIN *login, TDSLOCINFO *locale);
+TDSLOCINFO *tds_get_locale();
+void *tds_alloc_row(TDSRESULTINFO *res_info);
+char *tds_msg_get_proc_name(TDSSOCKET *tds);
+TDSLOGIN *tds_alloc_login();
+TDSDYNAMIC *tds_alloc_dynamic(TDSSOCKET *tds, char *id);
+void tds_free_login(TDSLOGIN *login);
+TDSCONFIGINFO *tds_alloc_config(TDSLOCINFO *locale);
+TDSLOCINFO *tds_alloc_locale();
+void tds_free_locale(TDSLOCINFO *locale);
+TDSSOCKET *tds_connect(TDSLOGIN *login, TDSCONTEXT *context, void *parent);
+TDSINPUTPARAM *tds_add_input_param(TDSDYNAMIC *dyn);
+void tds_set_packet(TDSLOGIN *tds_login, short packet_size);
+void tds_set_port(TDSLOGIN *tds_login, int port);
+void tds_set_passwd(TDSLOGIN *tds_login, char *password);
+void tds_set_bulk(TDSLOGIN *tds_login, TDS_TINYINT enabled);
+void tds_set_user(TDSLOGIN *tds_login, char *username);
+void tds_set_app(TDSLOGIN *tds_login, char *application);
+void tds_set_host(TDSLOGIN *tds_login, char *hostname);
+void tds_set_library(TDSLOGIN *tds_login, char *library);
+void tds_set_server(TDSLOGIN *tds_login, char *server);
+void tds_set_charset(TDSLOGIN *tds_login, char *charset);
+void tds_set_language(TDSLOGIN *tds_login, char *language);
+void tds_set_version(TDSLOGIN *tds_login, short major_ver, short minor_ver);
+void tds_set_capabilities(TDSLOGIN *tds_login, unsigned char *capabilities, int size);
+int tds_submit_query(TDSSOCKET *tds, char *query);
+int tds_process_result_tokens(TDSSOCKET *tds);
+int tds_process_row_tokens(TDSSOCKET *tds);
+int tds_process_env_chg(TDSSOCKET *tds);
+int tds_process_default_tokens(TDSSOCKET *tds, int marker);
+TDS_INT tds_process_end(TDSSOCKET *tds, int marker, int *more, int *canceled);
+int tds_client_msg(TDSCONTEXT *tds_ctx, TDSSOCKET *tds, int msgnum, int level, int state, int line, char *message);
+int tds_reset_msg_info(TDSMSGINFO *msg_info);
+void  tds_set_parent(TDSSOCKET* tds, void* the_parent);
+void* tds_get_parent(TDSSOCKET* tds);
+void tds_set_null(unsigned char *current_row, int column);
+void tds_clr_null(unsigned char *current_row, int column);
+int tds_get_null(unsigned char *current_row, int column);
+int tds7_send_login(TDSSOCKET *tds, TDSCONFIGINFO *config);
+unsigned char *tds7_crypt_pass(const unsigned char *clear_pass, int len, unsigned char *crypt_pass);
+int tds_lookup_dynamic(TDSSOCKET *tds, char *id);
+
+/* iconv.c */
+void tds_iconv_open(TDSSOCKET *tds, char *charset);
+void tds_iconv_close(TDSSOCKET *tds);
+unsigned char *tds7_ascii2unicode(TDSSOCKET *tds, const char *in_string, char *out_string, int maxlen);
+char *tds7_unicode2ascii(TDSSOCKET *tds, const char *in_string, char *out_string, int len);
  
+/* threadsafe.c */
+char *tds_timestamp_str(char *str, int maxlen);
+struct hostent *tds_gethostbyname_r(const char *servername, struct hostent *result, char *buffer, int buflen, int *h_errnop);
+struct hostent *tds_gethostbyaddr_r(const char *addr, int len, int type, struct hostent *result, char *buffer, int buflen, int *h_errnop);
+struct servent *tds_getservbyname_r(const char *name, char *proto, struct servent *result, char *buffer, int buflen);
+char *tds_strtok_r(char *s, const char *delim, char **ptrptr);
+
+typedef struct tds_answer
+{
+	unsigned char lm_resp[24];
+	unsigned char nt_resp[24];
+} TDSANSWER;
+void tds_answer_challenge(char *passwd, char *challenge,TDSANSWER* answer);
 
 #define IS_TDS42(x) (x->major_version==4 && x->minor_version==2)
 #define IS_TDS46(x) (x->major_version==4 && x->minor_version==6)
 #define IS_TDS50(x) (x->major_version==5 && x->minor_version==0)
 #define IS_TDS70(x) (x->major_version==7 && x->minor_version==0)
+#define IS_TDS80(x) (x->major_version==8 && x->minor_version==0)
 
+#define IS_TDS7_PLUS(x) ( IS_TDS70(x) || IS_TDS80(x) )
 
 #ifdef __cplusplus
 }
