@@ -20,10 +20,10 @@
 #ifndef _tdsver_h_
 #define _tdsver_h_
 
-#define VERSION_NO "freetds v0.52"
+#define TDS_VERSION_NO "freetds v0.60"
 
 static char  rcsid_tdsver_h [ ] =
-         "$Header: /Repository/freetds/include/tdsver.h.in,v 1.2 1999/06/13 00:07:17 camber Exp $";
+         "$Header: /cvsroot/freetds/freetds/include/tdsver.h.in,v 1.2 2002/05/25 00:33:49 brianb Exp $";
 static void *no_unused_tdsver_h_warn[]={rcsid_tdsver_h, no_unused_tdsver_h_warn};
 
 #endif
