@@ -17,21 +17,24 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <ctpublic.h>
 #include <ctlib.h>
+#include "tdsutil.h"
 
-static char  software_version[]   = "$Id: ct.c,v 1.63 2001/05/19 00:58:43 camber Exp $";
+static char  software_version[]   = "$Id: ct.c,v 1.26 2002/09/05 12:22:07 brianb Exp $";
 static void *no_unused_var_warn[] = {software_version,
                                      no_unused_var_warn};
 
-/* these function pointers are called by the tds layer to propagate message
-** back up to the CLI */
-extern int (*g_tds_msg_handler)();
-extern int (*g_tds_err_handler)();
 
-static void _ct_bind_data(CS_COMMAND *cmd);
+/**
+ * Read a row of data
+ * @return 0 on success
+ */
+static int _ct_bind_data(CS_COMMAND *cmd);
+static int _ct_get_client_type(int datatype, int size);
 
 
 CS_RETCODE ct_exit(CS_CONTEXT *ctx, CS_INT unused)
@@ -41,11 +44,11 @@ CS_RETCODE ct_exit(CS_CONTEXT *ctx, CS_INT unused)
 }
 CS_RETCODE ct_init(CS_CONTEXT *ctx, CS_INT version)
 {
+	/* uncomment the next line to get pre-login trace */
+	/* tdsdump_open("/tmp/tds2.log"); */
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_init()\n");
-   /* set the functions in the TDS layer to point to the correct info/err
-    * message handler functions */
-   g_tds_msg_handler = ctlib_handle_info_message;
-   g_tds_err_handler = ctlib_handle_err_message;
+	ctx->tds_ctx->msg_handler = ctlib_handle_info_message;
+	ctx->tds_ctx->err_handler = ctlib_handle_err_message;
 	return CS_SUCCEED;
 }
 CS_RETCODE ct_con_alloc(CS_CONTEXT *ctx, CS_CONNECTION **con)
@@ -66,6 +69,8 @@ CS_RETCODE ct_con_alloc(CS_CONTEXT *ctx, CS_CONNECTION **con)
 }
 CS_RETCODE ct_callback(CS_CONTEXT *ctx, CS_CONNECTION *con, CS_INT action, CS_INT type, CS_VOID *func)
 {
+	int	(*funcptr)(void*,void*,void*)=(int (*)(void*,void*,void*))func;
+
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_callback() action = %s\n",
 		CS_GET ? "CS_GET" : "CS_SET");
 	/* one of these has to be defined */
@@ -75,14 +80,14 @@ CS_RETCODE ct_callback(CS_CONTEXT *ctx, CS_CONNECTION *con, CS_INT action, CS_IN
 	if (action==CS_GET) {
 		switch(type) {
 			case CS_CLIENTMSG_CB:
-				func = (CS_VOID *) con ? con->_clientmsg_cb : ctx->_clientmsg_cb;
+				*(void **)func = (CS_VOID *) (con ? con->_clientmsg_cb : ctx->_clientmsg_cb);
 				return CS_SUCCEED;
 			case CS_SERVERMSG_CB:
-				func = (CS_VOID *) con ? con->_servermsg_cb : ctx->_servermsg_cb;
+				*(void **)func = (CS_VOID *) (con ? con->_servermsg_cb : ctx->_servermsg_cb);
 				return CS_SUCCEED;
 			default:
 				fprintf(stderr,"Unknown callback %d\n",type);
-				func = (CS_VOID *) NULL;
+				*(void **)func = (CS_VOID *) NULL;
 				return CS_SUCCEED;
 		}
 	}
@@ -90,15 +95,15 @@ CS_RETCODE ct_callback(CS_CONTEXT *ctx, CS_CONNECTION *con, CS_INT action, CS_IN
 	switch(type) {
 		case CS_CLIENTMSG_CB:
 			if (con) 
-				con->_clientmsg_cb = func;
+				con->_clientmsg_cb = funcptr;
 			else 
-				ctx->_clientmsg_cb = func;
+				ctx->_clientmsg_cb = funcptr;
 			break;
 		case CS_SERVERMSG_CB:
 			if (con) 
-				con->_servermsg_cb = func;
+				con->_servermsg_cb = funcptr;
 			else 
-				ctx->_servermsg_cb = func;
+				ctx->_servermsg_cb = funcptr;
 			break;
 	}
 	return CS_SUCCEED;
@@ -112,8 +117,8 @@ TDSSOCKET *tds;
 TDSLOGIN *tds_login;
 char *set_buffer = NULL;
 
-	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_con_props() action = %d property = %d\n",
-		action, property);
+	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_con_props() action = %s property = %d\n",
+		CS_GET ? "CS_GET" : "CS_SET", property);
 
 	tds = con->tds_socket;
 	tds_login = con->tds_login;
@@ -149,7 +154,7 @@ char *set_buffer = NULL;
 				tds_set_app(tds_login, set_buffer);
 				break;
 			case CS_HOSTNAME:
-				tds_set_app(tds_login, set_buffer);
+				tds_set_host(tds_login, set_buffer);
 				break;
 			case CS_LOC_PROP:
 				con->locale = (CS_LOCALE *)buffer;
@@ -159,6 +164,7 @@ char *set_buffer = NULL;
 					free(con->userdata);
 				}
 				con->userdata = (void *)malloc(buflen+1);
+				tdsdump_log(TDS_DBG_INFO2, "%L setting userdata orig %d new %d\n",buffer,con->userdata);
 				con->userdata_len = buflen;
 				memcpy(con->userdata, buffer, buflen);
 				break;
@@ -238,6 +244,7 @@ char *set_buffer = NULL;
 				buffer = (CS_VOID *)con->locale;
 				break;
 			case CS_USERDATA:
+				tdsdump_log(TDS_DBG_INFO2, "%L fetching userdata %d\n",con->userdata);
 				maxcp = con->userdata_len;
 				if (out_len) *out_len=maxcp;
 				if (maxcp>buflen) maxcp = buflen;
@@ -281,6 +288,7 @@ CS_RETCODE ct_connect(CS_CONNECTION *con, CS_CHAR *servername, CS_INT snamelen)
 {
 char *server;
 int needfree=0;
+CS_CONTEXT *ctx;
 
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_connect() servername = %s\n", servername);
 
@@ -295,13 +303,14 @@ int needfree=0;
 		server[snamelen]='\0';
 	}
         tds_set_server(con->tds_login,server);
-        if (!(con->tds_socket = (void *) tds_connect(con->tds_login))) {
+	   ctx = con->ctx;
+        if (!(con->tds_socket = (void *) tds_connect(con->tds_login, ctx->tds_ctx, (void *) con))) {
 		if (needfree) free(server);
 		tdsdump_log(TDS_DBG_FUNC, "%L leaving ct_connect() returning %d\n", CS_FAIL);
 		return CS_FAIL;
 	}
 
-        tds_set_parent( con->tds_socket, con);
+        /* tds_set_parent( con->tds_socket, con); */
 
 	if (needfree) free(server);
 
@@ -339,13 +348,57 @@ int query_len;
 
 	return CS_SUCCEED;
 }
+CS_RETCODE ct_send_dyn(CS_COMMAND *cmd)
+{
+	if (cmd->dynamic_cmd==CS_PREPARE) {
+		cmd->dynamic_cmd=0;
+		if (tds_submit_prepare(cmd->con->tds_socket, cmd->query, cmd->dyn_id)==TDS_FAIL)
+			return CS_FAIL;
+		else 
+			return CS_SUCCEED;
+	} else if (cmd->dynamic_cmd==CS_EXECUTE) {
+		cmd->dynamic_cmd=0;
+		if (tds_submit_execute(cmd->con->tds_socket, cmd->dyn_id)==TDS_FAIL)
+			return CS_FAIL;
+		else 
+			return CS_SUCCEED;
+	}
+	return CS_FAIL;
+}
 CS_RETCODE ct_send(CS_COMMAND *cmd)
 {
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_send()\n");
-	if (tds_submit_query(cmd->con->tds_socket, cmd->query)==TDS_FAIL)
+	if (cmd->dynamic_cmd) 
+		return ct_send_dyn(cmd);
+
+	if (tds_submit_query(cmd->con->tds_socket, cmd->query)==TDS_FAIL) {
+		tdsdump_log(TDS_DBG_WARN, "%L ct_send() failed\n");
 		return CS_FAIL;
-	else 
+	} else {
+		tdsdump_log(TDS_DBG_INFO2, "%L ct_send() succeeded\n");
 		return CS_SUCCEED;
+	}
+}
+CS_RETCODE ct_results_dyn(CS_COMMAND *cmd, CS_INT *result_type)
+{
+TDSRESULTINFO *resinfo;
+TDSSOCKET *tds;
+TDSDYNAMIC *dyn;
+
+	tds = cmd->con->tds_socket;
+
+	if (cmd->dynamic_cmd==CS_DESCRIBE_INPUT) {
+		dyn = tds->dyns[tds->cur_dyn_elem];
+		if (dyn->dyn_state) {
+			dyn->dyn_state = 0;
+			return CS_END_RESULTS;
+		} else {
+			dyn->dyn_state++;
+			*result_type = CS_DESCRIBE_RESULT;
+			return CS_SUCCEED;
+		}
+	}
+	return CS_FAIL;
 }
 CS_RETCODE ct_results(CS_COMMAND *cmd, CS_INT *result_type)
 {
@@ -354,12 +407,19 @@ TDSSOCKET *tds;
 int ret;
 
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_results()\n");
+
+	if (cmd->dynamic_cmd) {
+		return ct_results_dyn(cmd, result_type);
+	}
+
 	if (cmd->cmd_done) {
 		cmd->cmd_done = 0;
 		*result_type = CS_CMD_DONE;
 		return CS_SUCCEED;
 	} 
+
 	tds = cmd->con->tds_socket;
+
 	switch (ret = tds_process_result_tokens(tds)) {
 		case TDS_SUCCEED:
 			resinfo = tds->res_info;
@@ -371,6 +431,17 @@ int ret;
 			}
 			return CS_SUCCEED;
 		case TDS_NO_MORE_RESULTS:
+			if (! tds->res_info) {
+				if (cmd->empty_res_hack) {
+					cmd->empty_res_hack=0;
+					*result_type = CS_CMD_DONE;
+					return CS_END_RESULTS;
+				} else {
+					cmd->empty_res_hack=1;
+					*result_type = CS_CMD_SUCCEED;
+					return CS_SUCCEED;
+				}
+			}	
 			if (tds->res_info && !tds->res_info->rows_exist) {
 				if (cmd->empty_res_hack) {
 					cmd->empty_res_hack=0;
@@ -408,6 +479,7 @@ TDSSOCKET * tds;
 
    tds = (TDSSOCKET *) cmd->con->tds_socket;
    resinfo = tds->res_info;
+   /* FIXME check item value */
    colinfo = resinfo->columns[item-1];
    colinfo->varaddr = (char *)buffer;
    colinfo->column_bindtype = datafmt->datatype;
@@ -427,29 +499,34 @@ CS_RETCODE ct_fetch(CS_COMMAND *cmd, CS_INT type, CS_INT offset, CS_INT option,
 {
    tdsdump_log(TDS_DBG_FUNC, "%L inside ct_fetch()\n");
 
+   if (rows_read) *rows_read = 0;
    switch (tds_process_row_tokens(cmd->con->tds_socket)) {
       case TDS_SUCCEED:
+	if (_ct_bind_data(cmd))
+		return CS_ROW_FAIL;
       	if (rows_read) *rows_read = 1;
-      	_ct_bind_data(cmd);
       	return TDS_SUCCEED;
       case TDS_NO_MORE_ROWS:
-      	if (rows_read) *rows_read = 0;
       	return CS_END_DATA;
       default:
-      	if (rows_read) *rows_read = 0;
       	return CS_FAIL;
    }
    return CS_SUCCEED;
 }
 
-static void _ct_bind_data(CS_COMMAND *cmd)
+static int _ct_bind_data(CS_COMMAND *cmd)
 {
 int i;
 TDSCOLINFO *curcol;
-TDSRESULTINFO *resinfo = cmd->con->tds_socket->res_info;
+TDSSOCKET *tds = cmd->con->tds_socket;
+TDSRESULTINFO *resinfo = tds->res_info;
 unsigned char *src;
 unsigned char *dest;
+int result = 0;
 TDS_INT srctype, srclen, desttype, destlen, len;
+CS_CONTEXT *ctx = cmd->con->ctx;
+
+CS_DATAFMT srcfmt, destfmt;
 
    tdsdump_log(TDS_DBG_FUNC, "%L inside _ct_bind_data()\n");
 
@@ -467,68 +544,54 @@ TDS_INT srctype, srclen, desttype, destlen, len;
    
       srctype = curcol->column_type;
       desttype = _ct_get_server_type(curcol->column_bindtype);
-      dest = curcol->varaddr;
-      tdsdump_log(TDS_DBG_INFO1, "%L inside _ct_bind_data() converting column %d from type %d to type %d\n", 
-         i, srctype, desttype);
-   
+      dest = (unsigned char *)curcol->varaddr;
+  
       if (dest && !tds_get_null(resinfo->current_row,i)) {
-         srctype = tds_get_conversion_type(srctype, curcol->column_size);
-         destlen = curcol->column_bindlen;
-         if (is_blob_type(srctype)) {
-            src = curcol->column_textvalue;
+
+         srctype = _ct_get_client_type(curcol->column_type, curcol->column_size);
+
+         if (is_blob_type(curcol->column_type)) {
+            src = (unsigned char *)curcol->column_textvalue;
             srclen = curcol->column_textsize;
          } else {
             src = &(resinfo->current_row[curcol->column_offset]);
-            srclen = curcol->column_size;
-         }
-         tdsdump_log(TDS_DBG_INFO1, "%L inside _ct_bind_data() setting source length for %d = %d\n", i, srclen);
-         len = tds_convert(srctype, src, srclen, desttype, dest, destlen);
-         tdsdump_log(TDS_DBG_INFO2, "%L inside _ct_bind_data() conversion done len = %d bindfmt = %d\n", len, curcol->column_bindfmt);
-   
-         /* XXX need utf16 support (NCHAR,NVARCHAR,NTEXT) */
-         switch (curcol->column_bindfmt) {
-            case CS_FMT_NULLTERM:
-               if (desttype==SYBCHAR || desttype==SYBVARCHAR
-                || desttype==SYBTEXT) {
-                  if (destlen > len)  dest[len] = '\0';
-                  else                dest[len-1] = '\0';
-                  len++; /* CS_FMT_NULLTERM includes the null in the size */
-               }
-               break;      
-            case CS_FMT_UNUSED:
-               break;      
-            case CS_FMT_PADBLANK:
-               if (desttype==SYBCHAR || desttype==SYBVARCHAR
-                || desttype==SYBTEXT) {
-                  if (destlen > len) {
-                      memset(dest+len, (int)' ', destlen - len);
-                  }
-               }
-               break;      
-            case CS_FMT_PADNULL:
-               if (desttype==SYBCHAR   || desttype==SYBVARCHAR
-                || desttype==SYBBINARY || desttype==SYBVARBINARY
-                || desttype==SYBTEXT   || desttype==SYBIMAGE) {
-                  if (destlen > len) {
-                      memset(dest+len, 0, destlen - len);
-                  }
-               }
-               break;      
-            default:
-               break;
+	    srclen = curcol->cur_row_size;
          }
+
+         tdsdump_log(TDS_DBG_INFO1, "%L inside _ct_bind_data() setting source length for %d = %d destlen = %d\n", i, srclen, curcol->column_bindlen);
+
+         srcfmt.datatype  = srctype;
+         srcfmt.maxlength = srclen;
+         srcfmt.locale    = cmd->con->locale;
+
+         destfmt.datatype  = curcol->column_bindtype;
+         destfmt.maxlength = curcol->column_bindlen;
+         destfmt.locale    = cmd->con->locale;
+         destfmt.format    = curcol->column_bindfmt;
+
+	 /* if convert return FAIL mark error but process other columns */
+         if (cs_convert(ctx, &srcfmt, (CS_VOID *)src, &destfmt, (CS_VOID *)dest, &len) != CS_SUCCEED) {
+		 result = 1;
+		 len = 0;
+	 }
+
          if (curcol->column_lenbind) {
             tdsdump_log(TDS_DBG_INFO1, "%L inside _ct_bind_data() length binding len = %d\n", len);
             *((CS_INT *)curcol->column_lenbind) = len;
          }
+
       }
    }
+   return result;
 }
 
 CS_RETCODE ct_cmd_drop(CS_COMMAND *cmd)
 {
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_cmd_drop()\n");
-	if (cmd) free(cmd);
+	if (cmd) {
+		if (cmd->query) free(cmd->query);
+		free(cmd);
+	}
 	return CS_SUCCEED;
 }
 CS_RETCODE ct_close(CS_CONNECTION *con, CS_INT option)
@@ -551,9 +614,10 @@ CS_RETCODE ct_con_drop(CS_CONNECTION *con)
 }
 
 
-int _ct_get_client_type(int datatype, int size)
+static int _ct_get_client_type(int datatype, int size)
 {
-   tdsdump_log(TDS_DBG_FUNC, "%L inside _ct_get_client_type()\n");
+   tdsdump_log(TDS_DBG_FUNC, "%L inside _ct_get_client_type(type %d, size %d)\n",
+                             datatype, size);
    switch (datatype) {
          case SYBBIT:
          case SYBBITN:
@@ -624,7 +688,6 @@ int _ct_get_client_type(int datatype, int size)
 			return CS_DATETIME_TYPE;
 		} else {
 			fprintf(stderr,"Error! unknown date size of %d\n",size);
-			return CS_FAIL;
 		}
 		break;
          case SYBNUMERIC:
@@ -645,11 +708,16 @@ int _ct_get_client_type(int datatype, int size)
          case SYBTEXT:
       		return CS_TEXT_TYPE;
 		break;
+         case SYBUNIQUE:
+      		return CS_UNIQUE_TYPE;
+		break;
    }
+
+   return CS_FAIL;
 }
 int _ct_get_server_type(int datatype)
 {
-   tdsdump_log(TDS_DBG_FUNC, "%L inside _ct_get_server_type()\n");
+   tdsdump_log(TDS_DBG_FUNC, "%L inside _ct_get_server_type(%d)\n", datatype);
    switch (datatype) {
       case CS_IMAGE_TYPE:
 	 return SYBIMAGE;
@@ -702,6 +770,9 @@ int _ct_get_server_type(int datatype)
       case CS_TEXT_TYPE:
          return SYBTEXT;
 	 break;
+      case CS_UNIQUE_TYPE:
+         return SYBUNIQUE;
+	 break;
       default:
          return -1;
          break;
@@ -722,7 +793,7 @@ CS_RETCODE ct_cancel(CS_CONNECTION *conn, CS_COMMAND *cmd, CS_INT type)
          }
       }
       if (cmd->con->tds_socket) {
-         tds_free_results(cmd->con->tds_socket->res_info);
+         tds_free_all_results(cmd->con->tds_socket);
       }
       return ret;
    }
@@ -738,10 +809,19 @@ CS_RETCODE ct_cancel(CS_CONNECTION *conn, CS_COMMAND *cmd, CS_INT type)
 
 CS_RETCODE ct_describe(CS_COMMAND *cmd, CS_INT item, CS_DATAFMT *datafmt)
 {
-TDSRESULTINFO *resinfo = cmd->con->tds_socket->res_info;
+TDSSOCKET *tds;
+TDSRESULTINFO *resinfo;
 TDSCOLINFO *curcol;
 
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_describe()\n");
+	tds = cmd->con->tds_socket;
+
+	if (cmd->dynamic_cmd) {
+		resinfo = tds->dyns[tds->cur_dyn_elem]->res_info;
+	} else {
+ 		resinfo = cmd->con->tds_socket->res_info;
+	}
+
 	if (item<1 || item>resinfo->num_cols) return CS_FAIL;	
 	curcol=resinfo->columns[item-1];
 	strncpy(datafmt->name, curcol->column_name, CS_MAX_NAME);
@@ -762,19 +842,50 @@ TDSCOLINFO *curcol;
 	
 	return CS_SUCCEED;
 }
+CS_RETCODE ct_res_info_dyn(CS_COMMAND *cmd, CS_INT type, CS_VOID *buffer, CS_INT buflen, CS_INT *out_len)
+{
+TDSSOCKET *tds = cmd->con->tds_socket;
+TDSDYNAMIC *dyn;
+CS_INT int_val;
+
+	switch(type) {
+		case CS_NUMDATA:
+			dyn = tds->dyns[tds->cur_dyn_elem];
+			int_val = dyn->res_info->num_cols;
+			memcpy(buffer, &int_val, sizeof(CS_INT));
+			break;
+		default:
+			fprintf(stderr,"Unknown type in ct_res_info_dyn: %d\n",type);
+			return CS_FAIL;
+	}
+	return CS_SUCCEED;
+}
 CS_RETCODE ct_res_info(CS_COMMAND *cmd, CS_INT type, CS_VOID *buffer, CS_INT buflen, CS_INT *out_len)
 {
-TDSRESULTINFO *resinfo = cmd->con->tds_socket->res_info;
+TDSSOCKET *tds = cmd->con->tds_socket;
+TDSRESULTINFO *resinfo = tds->res_info;
 CS_INT int_val;
 
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_res_info()\n");
+	if (cmd->dynamic_cmd) {
+		return ct_res_info_dyn(cmd, type, buffer, buflen, out_len);
+	}
 	switch(type) {
 		case CS_NUMDATA:
-			int_val = resinfo->num_cols;
+			if (!resinfo) {
+				int_val = 0;
+			} else {
+				int_val = resinfo->num_cols;
+			}
 			memcpy(buffer, &int_val, sizeof(CS_INT));
 			break;
 		case CS_ROW_COUNT:
-			int_val = resinfo->row_count;
+			/* resinfo check by kostya@warmcat.excom.spb.su */
+			if (resinfo) {
+				int_val = resinfo->row_count;
+			} else {
+				int_val = tds->rows_affected;
+			}
 			memcpy(buffer, &int_val, sizeof(CS_INT));
 			break;
 		default:
@@ -786,12 +897,14 @@ CS_INT int_val;
 }
 CS_RETCODE ct_config(CS_CONTEXT *ctx, CS_INT action, CS_INT property, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen)
 {
-	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_config()\n");
+	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_config() action = %s property = %d\n",
+		CS_GET ? "CS_GET" : "CS_SET", property);
 	return CS_SUCCEED;
 }
 CS_RETCODE ct_cmd_props(CS_COMMAND *cmd, CS_INT action, CS_INT property, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen)
 {
-	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_cmd_props()\n");
+	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_cmd_props() action = %s property = %d\n",
+		CS_GET ? "CS_GET" : "CS_SET", property);
 	return CS_SUCCEED;
 }
 CS_RETCODE ct_compute_info(CS_COMMAND *cmd, CS_INT type, CS_INT colnum, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen)
@@ -804,6 +917,17 @@ CS_RETCODE ct_get_data(CS_COMMAND *cmd, CS_INT item, CS_VOID *buffer, CS_INT buf
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_get_data()\n");
 	return CS_SUCCEED;
 }
+CS_RETCODE ct_send_data(CS_COMMAND *cmd, CS_VOID *buffer, CS_INT buflen)
+{
+	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_send_data()\n");
+	return CS_SUCCEED;
+}
+CS_RETCODE ct_data_info(CS_COMMAND *cmd, CS_INT action, CS_INT colnum,
+CS_IODESC *iodesc)
+{
+	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_data_info()\n");
+	return CS_SUCCEED;
+}
 CS_RETCODE ct_capability(CS_CONNECTION *con, CS_INT action, CS_INT type, CS_INT capability, CS_VOID *value)
 {
 TDSLOGIN *login;
@@ -1139,7 +1263,8 @@ unsigned char *mask;
 		case CS_REQ_CURSOR:
 			*((CS_BOOL *)value)=mask[8]&0x40 ? CS_TRUE : CS_FALSE;break;
 		case CS_REQ_DYN:
-			*((CS_BOOL *)value)=CS_FALSE; /* mask[8]&0x80 ? CS_TRUE : CS_FALSE;*/ break;
+			*((CS_BOOL *)value)=mask[8]&0x80 ? CS_TRUE : CS_FALSE;break;
+			/* *((CS_BOOL *)value)=CS_FALSE; */
 		}
 	} else {
 		/* bad values */
@@ -1150,15 +1275,57 @@ unsigned char *mask;
 CS_RETCODE ct_dynamic(CS_COMMAND *cmd, CS_INT type, CS_CHAR *id, CS_INT idlen, CS_CHAR *buffer, CS_INT buflen)
 {
 static int stmt_no=1;
+int query_len, id_len;
+TDSDYNAMIC *dyn;
+TDSSOCKET *tds;
+int elem;
 
+	cmd->dynamic_cmd=type;
 	switch(type) {
 		case CS_PREPARE:
+			/* store away the id */
+		     if (idlen==CS_NULLTERM) {
+				id_len = strlen(id);
+        		} else {
+				id_len = idlen;
+			}
+			if (cmd->dyn_id) free(cmd->dyn_id);
+			cmd->dyn_id = (char *) malloc(id_len + 1);
+			strncpy(cmd->dyn_id,(char *)id,id_len);
+			cmd->dyn_id[id_len]='\0';
+
+			/* now the query */
+		        if (buflen==CS_NULLTERM) {
+				query_len = strlen(buffer);
+        		} else {
+				query_len = buflen;
+			}
+			if (cmd->query) free(cmd->query);
+			cmd->query = (char *) malloc(query_len + 1);
+			strncpy(cmd->query,(char *)buffer,query_len);
+			cmd->query[query_len]='\0';
+
 			break;
 		case CS_DEALLOC:
 			break;
 		case CS_DESCRIBE_INPUT:
 			break;
 		case CS_EXECUTE:
+			/* store away the id */
+		     if (idlen==CS_NULLTERM) {
+				id_len = strlen(id);
+        		} else {
+				id_len = idlen;
+			}
+			if (cmd->dyn_id) free(cmd->dyn_id);
+			cmd->dyn_id = (char *) malloc(id_len + 1);
+			strncpy(cmd->dyn_id,(char *)id,id_len);
+			cmd->dyn_id[id_len]='\0';
+
+			/* free any input parameters */
+			tds = cmd->con->tds_socket;
+			elem = tds_lookup_dynamic(tds, cmd->dyn_id);
+			dyn = tds->dyns[elem];
 			break;
 	}
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_dynamic()\n");
@@ -1166,10 +1333,33 @@ static int stmt_no=1;
 }
 CS_RETCODE ct_param(CS_COMMAND *cmd, CS_DATAFMT *datafmt, CS_VOID *data, CS_INT datalen, CS_SMALLINT indicator)
 {
+TDSSOCKET *tds;
+TDSDYNAMIC *dyn;
+TDSINPUTPARAM *param;
+int elem;
+
 	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_param()\n");
+	tdsdump_log(TDS_DBG_INFO1, "%L ct_param() data addr = %d data length = %d\n", data, datalen);
+
+	tds = cmd->con->tds_socket;
+
+	elem = tds_lookup_dynamic(tds, cmd->dyn_id);
+	dyn = tds->dyns[elem];
+	param = tds_add_input_param(dyn);
+	param->column_type = _ct_get_server_type(datafmt->datatype);
+	param->varaddr = data;
+	if (datalen==CS_NULLTERM) {
+		param->column_bindlen = 0;
+	} else {
+		param->column_bindlen = datalen;
+	}
+	param->is_null = indicator;
+
+	return CS_SUCCEED;
 }
 CS_RETCODE ct_options(CS_CONNECTION *con, CS_INT action, CS_INT option, CS_VOID *param, CS_INT paramlen, CS_INT *outlen)
 {
-	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_options()\n");
+	tdsdump_log(TDS_DBG_FUNC, "%L inside ct_options() action = %s option = %d\n",
+		CS_GET ? "CS_GET" : "CS_SET", option);
 	return CS_SUCCEED;
 }
