@@ -11,7 +11,7 @@
 
 #include "common.h"
 
-static char  software_version[]   = "$Id: t0001.c,v 1.1.1.1 2001/10/12 23:29:10 brianb Exp $";
+static char  software_version[]   = "$Id: t0001.c,v 1.2 2002/08/29 09:54:54 freddy77 Exp $";
 static void *no_unused_var_warn[] = {software_version,
                                      no_unused_var_warn};
 
@@ -71,7 +71,7 @@ fprintf(stdout, "About to open\n");
 
    fprintf(stdout, "Dropping table\n");
    add_bread_crumb();
-   dbcmd(dbproc, "drop table dblib0001");
+   dbcmd(dbproc, "drop table #dblib0001");
    add_bread_crumb();
    dbsqlexec(dbproc);
    add_bread_crumb();
@@ -83,7 +83,7 @@ fprintf(stdout, "About to open\n");
 
    fprintf(stdout, "creating table\n");
    dbcmd(dbproc,
-         "create table dblib0001 (i int not null, s char(10) not null)");
+         "create table #dblib0001 (i int not null, s char(10) not null)");
    dbsqlexec(dbproc);
    while (dbresults(dbproc)==SUCCEED)
    {
@@ -95,7 +95,7 @@ fprintf(stdout, "About to open\n");
    {
       char   cmd[1024];
 
-      sprintf(cmd, "insert into dblib0001 values (%d, 'row %03d')", i, i);
+      sprintf(cmd, "insert into #dblib0001 values (%d, 'row %03d')", i, i);
       fprintf(stdout, "%s\n",cmd);
       dbcmd(dbproc, cmd);
       dbsqlexec(dbproc);
@@ -106,7 +106,7 @@ fprintf(stdout, "About to open\n");
    }
 
    fprintf(stdout, "select\n");
-   dbcmd(dbproc,"select * from dblib0001 order by i");
+   dbcmd(dbproc,"select * from #dblib0001 order by i");
    dbsqlexec(dbproc);
    add_bread_crumb();
 
@@ -151,6 +151,7 @@ fprintf(stdout, "About to open\n");
       sprintf(expected, "row %03d", i);
 
       add_bread_crumb();
+      teststr[0] = 0;
       if (REG_ROW != dbnextrow(dbproc))
       {
          failed = 1;
