@@ -12,7 +12,7 @@
 #include "common.h"
 
 
-static char  software_version[]   = "$Id: t0004.c,v 1.1.1.1 2001/10/12 23:29:11 brianb Exp $";
+static char  software_version[]   = "$Id: t0004.c,v 1.2 2002/08/29 09:54:54 freddy77 Exp $";
 static void *no_unused_var_warn[] = {software_version,
                                      no_unused_var_warn};
 
@@ -69,7 +69,7 @@ fprintf(stdout, "About to open\n");
    
    fprintf(stdout, "Dropping table\n");
    add_bread_crumb();
-   dbcmd(dbproc, "drop table dblib0004");
+   dbcmd(dbproc, "drop table #dblib0004");
    add_bread_crumb();
    dbsqlexec(dbproc);
    add_bread_crumb();
@@ -81,7 +81,7 @@ fprintf(stdout, "About to open\n");
    
    fprintf(stdout, "creating table\n");
    dbcmd(dbproc,
-         "create table dblib0004 (i int not null, s char(10) not null)");
+         "create table #dblib0004 (i int not null, s char(10) not null)");
    dbsqlexec(dbproc);
    while (dbresults(dbproc)!=NO_MORE_RESULTS)
    {
@@ -91,7 +91,7 @@ fprintf(stdout, "About to open\n");
    fprintf(stdout, "insert\n");
    for(i=1; i<rows_to_add; i++)
    {
-      sprintf(cmd, "insert into dblib0004 values (%d, 'row %04d')", i, i);
+      sprintf(cmd, "insert into #dblib0004 values (%d, 'row %04d')", i, i);
       fprintf(stdout, "%s\n",cmd);
       dbcmd(dbproc, cmd);
       dbsqlexec(dbproc);
@@ -101,7 +101,7 @@ fprintf(stdout, "About to open\n");
       }
    }
 
-   sprintf(cmd, "select * from dblib0004 where i<=25 order by i");
+   sprintf(cmd, "select * from #dblib0004 where i<=25 order by i");
    fprintf(stdout, "%s\n", cmd);
    dbcmd(dbproc, cmd);
    dbsqlexec(dbproc);
@@ -173,7 +173,7 @@ fprintf(stdout, "About to open\n");
 
    fprintf(stdout, "second select\n");
    
-   sprintf(cmd, "select * from dblib0004 where i>950 order by i");
+   sprintf(cmd, "select * from #dblib0004 where i>950 order by i");
    fprintf(stdout, "%s\n", cmd);
    if (SUCCEED != dbcmd(dbproc, cmd))
    {
