@@ -14,13 +14,14 @@
 
 
 
-static char  software_version[]   = "$Id: t0013.c,v 1.1.1.1 2001/10/12 23:29:13 brianb Exp $";
+static char  software_version[]   = "$Id: t0013.c,v 1.4 2002/08/31 06:32:44 freddy77 Exp $";
 static void *no_unused_var_warn[] = {software_version,
                                      no_unused_var_warn};
 #define BLOB_BLOCK_SIZE 4096
 
 int failed = 0;
 
+char *testargs[] = { "", "data.bin", "t0013.out" };
 
 int main(int argc, char *argv[])
 {
@@ -69,7 +70,7 @@ int main(int argc, char *argv[])
    DBSETLAPP(login,"t0013");
    
    fprintf(stdout, "About to open, PASSWORD: %s, USER: %s, SERVER: %s\n",
-   	"","",""); // PASSWORD, USER, SERVER);
+   	"","",""); /* PASSWORD, USER, SERVER); */
 
    dbproc = dbopen(login, SERVER);
    if (strlen(DATABASE)) {
@@ -78,7 +79,11 @@ int main(int argc, char *argv[])
    fprintf(stdout, "After logon\n");
 
   fprintf(stdout, "About to read binary input file\n");
-  
+
+  if (argc == 1) {
+	  argv = testargs;
+	  argc = 3;
+  }  
   if (argc < 3) {
 	 fprintf(stderr, "Usage: %s infile outfile\n", argv[0]);
 	 return 1;
@@ -97,7 +102,7 @@ int main(int argc, char *argv[])
   fclose (fp);
 
   fprintf(stdout, "Dropping table\n");
-  dbcmd(dbproc, "drop table dblib0013");
+  dbcmd(dbproc, "drop table #dblib0013");
   dbsqlexec(dbproc);
   while (dbresults(dbproc)!=NO_MORE_RESULTS)
   {
@@ -106,7 +111,7 @@ int main(int argc, char *argv[])
 
    fprintf(stdout, "creating table\n");
    dbcmd(dbproc,
-         "create table dblib0013 (i int not null, PigTure image not null)");
+         "create table #dblib0013 (i int not null, PigTure image not null)");
    dbsqlexec(dbproc);
    while (dbresults(dbproc)!=NO_MORE_RESULTS)
    {
@@ -116,7 +121,7 @@ int main(int argc, char *argv[])
 
    fprintf(stdout, "insert\n");
 
-   sprintf(cmd, "insert into dblib0013 values (1, '')");
+   sprintf(cmd, "insert into #dblib0013 values (1, '')");
    fprintf(stdout, "%s\n",cmd);
    dbcmd(dbproc, cmd);
    dbsqlexec(dbproc);
@@ -125,7 +130,7 @@ int main(int argc, char *argv[])
       /* nop */
    }
 
-   sprintf(sqlCmd, "SELECT PigTure FROM dblib0013 WHERE i = 1");
+   sprintf(sqlCmd, "SELECT PigTure FROM #dblib0013 WHERE i = 1");
    dbcmd(dbproc, sqlCmd); 
    dbsqlexec(dbproc);			 
    if (dbresults(dbproc) != SUCCEED) {
@@ -136,13 +141,13 @@ int main(int argc, char *argv[])
    while ((result = dbnextrow(dbproc)) != NO_MORE_ROWS) {
    	result = REG_ROW ;
    	result = DBTXPLEN;
-   	strcpy(objname, "dblib0013.PigTure");
+   	strcpy(objname, "#dblib0013.PigTure");
    	textPtr = dbtxptr(dbproc, 1);
    	timeStamp = dbtxtimestamp(dbproc, 1);
    }
    
-   // Use #ifdef if you want to test dbmoretext mode (needed for 16-bit apps)
-   // Use #ifndef for big buffer version (32-bit)
+   /* Use #ifdef if you want to test dbmoretext mode (needed for 16-bit apps)
+      Use #ifndef for big buffer version (32-bit) */
 #ifndef DBWRITE_OK_FOR_OVER_4K
    if (dbwritetext(dbproc, objname, textPtr, DBTXPLEN, timeStamp, FALSE, isiz, blob) != SUCCEED)
    return 5;
@@ -153,7 +158,7 @@ int main(int argc, char *argv[])
    dbresults(dbproc);
 
    numtowrite = 0;
-   // Send the update value in chunks. 
+   /* Send the update value in chunks. */
    for (numwritten = 0; numwritten < isiz; numwritten += numtowrite) {
       numtowrite = (isiz - numwritten);
       if (numtowrite > BLOB_BLOCK_SIZE)
@@ -167,7 +172,7 @@ int main(int argc, char *argv[])
 
    fprintf(stdout, "select\n");
 
-   dbcmd(dbproc,"select * from dblib0013 order by i");
+   dbcmd(dbproc,"select * from #dblib0013 order by i");
    dbsqlexec(dbproc);
 
    if (dbresults(dbproc)!=SUCCEED)
@@ -204,7 +209,7 @@ int main(int argc, char *argv[])
    }
    dbnextrow(dbproc);
 
-   // get the image
+   /* get the image */
    strcpy(sqlCmd, "SET TEXTSIZE 2147483647");
    dbcmd(dbproc, sqlCmd); 
    dbsqlexec(dbproc);			 
@@ -212,7 +217,7 @@ int main(int argc, char *argv[])
 			
    fprintf(stdout, "select 2\n");
 
-   sprintf(sqlCmd, "SELECT PigTure FROM dblib0013 WHERE i = 1");
+   sprintf(sqlCmd, "SELECT PigTure FROM #dblib0013 WHERE i = 1");
    dbcmd(dbproc, sqlCmd); 
    dbsqlexec(dbproc);			 
    if (dbresults(dbproc) != SUCCEED) {
@@ -225,7 +230,7 @@ int main(int argc, char *argv[])
    readFirstImage = FALSE;
    while ((result = dbreadtext(dbproc, rbuf, BLOB_BLOCK_SIZE)) != NO_MORE_ROWS)
    {
-       if (result == 0) // this indicates end of row
+       if (result == 0) /* this indicates end of row */
        {
           readFirstImage = TRUE;
        } else {
