@@ -17,11 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "tds.h"
 #include "tdsutil.h"
 #include <unistd.h>
 
-static char  software_version[]   = "$Id: query.c,v 1.2 1999/06/13 00:07:18 camber Exp $";
+static char  software_version[]   = "$Id: query.c,v 1.3 2002/01/31 02:21:44 brianb Exp $";
 static void *no_unused_var_warn[] = {software_version,
                                      no_unused_var_warn};
 
@@ -36,5 +37,5 @@ int len;
 	len = tds_get_smallint(tds); /* query size +1 */
 	tds_get_n(tds,NULL,3);
 	tds_get_n(tds, query, len - 1);
-	return query;
+	return (char *)query;
 }
