@@ -17,22 +17,18 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "tds.h"
 #include "tdsutil.h"
+#include "tdsconvert.h"
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
 
-static char  software_version[]   = "$Id: token.c,v 1.78 2001/03/21 03:22:05 camber Exp $";
+static char  software_version[]   = "$Id: token.c,v 1.47 2002/09/06 21:13:27 castellano Exp $";
 static void *no_unused_var_warn[] = {software_version,
                                      no_unused_var_warn};
 
-/* 
-** All these functions process the input from the wire so it 
-** may be used by the upper level functions.
-*/
-
-int (*g_tds_msg_handler)() = NULL;
-int (*g_tds_err_handler)() = NULL;
-
-
 static int tds_process_msg(TDSSOCKET *tds,int marker);
 static int tds_process_compute_result(TDSSOCKET *tds);
 static int tds_process_result(TDSSOCKET *tds);
@@ -43,13 +39,19 @@ static int tds_process_row(TDSSOCKET *tds);
 static int tds_process_param_result(TDSSOCKET *tds);
 static int tds7_process_result(TDSSOCKET *tds);
 static int tds_process_param_result_tokens(TDSSOCKET *tds);
+static void tds_process_dyn_result(TDSSOCKET *tds);
+static int tds_process_dynamic(TDSSOCKET *tds);
+static int tds_process_auth(TDSSOCKET *tds);
+static int tds_get_varint_size(int datatype);
+static int tds_get_cardinal_type(int datatype);
+void tds_swap_datatype(int coltype, unsigned char *buf);
 
 /*
 ** The following little table is indexed by precision and will
 ** tell us the number of bytes required to store the specified
 ** precision.
 */
-extern int g__numeric_bytes_per_prec[];
+extern const int g__numeric_bytes_per_prec[];
 
 
 /*
@@ -72,7 +74,10 @@ int   cancelled;
    }
 
    switch(marker) {
-      case TDS_ENV_CHG_TOKEN:
+	case TDS_AUTH_TOKEN:
+		tds_process_auth(tds);
+		break;
+	case TDS_ENV_CHG_TOKEN:
       {
          tds_process_env_chg(tds);
          break;
@@ -142,8 +147,14 @@ int   cancelled;
       case TDS_ROW_TOKEN:
          tds_process_row(tds); 
          break;
+      case TDS5_DYN_TOKEN:
+      case TDS5_DYNRES_TOKEN:
+      case TDS5_DYN3_TOKEN:
+	 tdsdump_log(TDS_DBG_WARN, "eating token %d\n",marker);
+         tds_get_n(tds, NULL, tds_get_smallint(tds));
+         break;
       default:
-         fprintf(stderr,"Unknown marker: %d!!\n",marker); 
+	 tdsdump_log(TDS_DBG_ERROR, "Unknown marker: %d(%x)!!\n",marker,(unsigned char)marker); 
          return TDS_FAIL;
    }	
    return TDS_SUCCEED;
@@ -171,6 +182,9 @@ char *tmpbuf;
 	do {
 		marker=tds_get_byte(tds);
 		switch(marker) {
+			case TDS_AUTH_TOKEN:
+				tds_process_auth(tds);
+				break;
 			case TDS_LOGIN_ACK_TOKEN:
 				len = tds_get_smallint(tds);
 				ack = tds_get_byte(tds);
@@ -179,9 +193,11 @@ char *tmpbuf;
 				tds_get_n(tds, NULL, len-4);
 				tds_get_byte(tds);
 #ifdef WORDS_BIGENDIAN
+/*
 				if (major_ver==7) {
 					tds->broken_dates=1;
 				}
+*/
 #endif
 /*
 				tmpbuf = (char *) malloc(len);
@@ -203,6 +219,48 @@ char *tmpbuf;
 	tdsdump_log(TDS_DBG_FUNC, "%L leaving tds_process_login_tokens() returning %d\n",succeed);
 	return succeed;
 }
+
+static int tds_process_auth(TDSSOCKET *tds)
+{
+int pdu_size, ntlm_size;
+char nonce[10];
+/* char domain[30]; */
+int where = 0;
+
+	pdu_size = tds_get_smallint(tds);
+	tdsdump_log(TDS_DBG_INFO1, "TDS_AUTH_TOKEN PDU size %d\n", pdu_size);
+
+	tds_get_n(tds, NULL, 8); /* NTLMSSP\0 */ 
+	where += 8;
+	tds_get_int(tds); /* sequence -> 2 */
+	where += 4;
+	tds_get_n(tds, NULL, 4); /* domain len (2 time) */
+	where += 4;
+	ntlm_size = tds_get_int(tds); /* size of remainder of ntlmssp packet */
+	where += 4;
+	tdsdump_log(TDS_DBG_INFO1, "TDS_AUTH_TOKEN NTLMSSP size %d\n", ntlm_size);
+	tds_get_n(tds, NULL, 4); /* flags */
+	where += 4;
+	tds_get_n(tds, nonce, 8); 
+	where += 8;
+	tdsdump_log(TDS_DBG_INFO1, "TDS_AUTH_TOKEN nonce\n");
+	tdsdump_dump_buf(nonce, 8);
+	tds_get_n(tds, NULL, 8); 
+	where += 8;
+	
+	/*
+	tds_get_ntstring(tds, domain, 30); 
+	tdsdump_log(TDS_DBG_INFO1, "TDS_AUTH_TOKEN domain %s\n", domain);
+	where += strlen(domain);
+	*/
+
+	tds_get_n(tds, NULL, pdu_size - where); 
+	tdsdump_log(TDS_DBG_INFO1,"%L Draining %d bytes\n", pdu_size - where);
+
+	tds7_send_auth(tds, nonce);
+
+	return TDS_SUCCEED;
+}
 /*
 ** tds_process_result_tokens() is called after submitting a query with
 ** tds_submit_query() and is responsible for calling the routines to
@@ -215,6 +273,7 @@ int marker;
 int more_results = 0;
 int cancelled;
 int retcode=TDS_NO_MORE_RESULTS;
+int rc;
 
 	if (tds->state==TDS_COMPLETED) {
 		/* tds_process_row() now eats the end marker and sets
@@ -232,10 +291,9 @@ tdsdump_log(TDS_DBG_INFO1, "%L processing result tokens.  marker is  %x\n", mark
       			case TDS_ERR_TOKEN:
       			case TDS_MSG_TOKEN:
       			case TDS_EED_TOKEN:
-				tds_process_msg(tds,marker);
-				if (tds->msg_info &&
-				    tds->msg_info->priv_msg_type==1)
-					/* don't fail until we get a DONE */
+				rc = tds_process_msg(tds,marker);
+				/* don't fail until we get a DONE */
+				if (rc == TDS_ERROR) 
 					retcode=TDS_FAIL;
 				break;
 			case TDS7_RESULT_TOKEN:
@@ -277,6 +335,12 @@ tdsdump_log(TDS_DBG_INFO1, "%L processing result tokens.  marker is  %x\n", mark
 				tds->ret_status=tds_get_int(tds);
 				/* return TDS_SUCCEED; */
 				break;
+			case TDS5_DYN_TOKEN:
+				tds->cur_dyn_elem = tds_process_dynamic(tds);
+				break;
+      		case TDS5_DYNRES_TOKEN:
+				tds_process_dyn_result(tds);
+				break;
 			case TDS_DONE_TOKEN:
 			case TDS_DONEPROC_TOKEN:
 			case TDS_DONEINPROC_TOKEN:
@@ -313,7 +377,7 @@ int   cancelled;
 
 	while (1) {
 		marker=tds_get_byte(tds);
-tdsdump_log(TDS_DBG_INFO1, "%L processing row tokens.  marker is  %x\n", marker);
+        tdsdump_log(TDS_DBG_INFO1, "%L processing row tokens.  marker is  %x\n", marker);
 		switch(marker) {
 			case TDS_RESULT_TOKEN:
 			case TDS7_RESULT_TOKEN:
@@ -336,7 +400,6 @@ tdsdump_log(TDS_DBG_INFO1, "%L processing row tokens.  marker is  %x\n", marker)
 				break;
 		}
 	} 
-	/* make lint happy */
 	return TDS_SUCCEED;
 }
 
@@ -350,9 +413,9 @@ static int tds_process_col_name(TDSSOCKET *tds)
 {
 int hdrsize, len=0;
 int col,num_cols=0;
-int colnamelen;
 struct tmp_col_struct {
 	char *column_name;
+	int column_namelen;
 	struct tmp_col_struct *next;
 };
 struct tmp_col_struct *head=NULL, *cur=NULL, *prev;
@@ -373,13 +436,13 @@ TDSRESULTINFO *info;
 		if (prev) prev->next=cur;
 		if (!head) head = cur;
 
-		colnamelen = tds_get_byte(tds);
-		cur->column_name = (char *) malloc(colnamelen+1);
-		tds_get_n(tds,cur->column_name, colnamelen);
-		cur->column_name[colnamelen]='\0';
+		cur->column_namelen = tds_get_byte(tds);
+		cur->column_name = (char *) malloc(cur->column_namelen+1);
+		tds_get_n(tds,cur->column_name, cur->column_namelen);
+		cur->column_name[cur->column_namelen]='\0';
 		cur->next=NULL;
 
-		len += colnamelen + 1;
+		len += cur->column_namelen + 1;
 		num_cols++;
 	}
 
@@ -394,6 +457,7 @@ TDSRESULTINFO *info;
 	for (col=0;col<info->num_cols;col++) 
 	{
 		curcol=info->columns[col];
+		curcol->column_namelen = cur->column_namelen;
 		strncpy(curcol->column_name, cur->column_name, 
 			sizeof(curcol->column_name));
 		prev=cur; cur=cur->next;
@@ -417,10 +481,11 @@ TDSRESULTINFO *info;
 TDS_SMALLINT tabnamesize;
 int bytes_read = 0;
 int rest;
-/* XXX this 4 should be a machine dependent #define */
-int align = 4;
+/* XXX do a best check for alignment than this */
+union { void *p; int i; } align_struct;
+const int align = sizeof(align_struct);
 int remainder;
-
+char ci_flags[4];
 
 	hdrsize = tds_get_smallint(tds);
 
@@ -428,20 +493,37 @@ int remainder;
 	for (col=0; col<info->num_cols; col++) 
 	{
 		curcol=info->columns[col];
-		tds_get_n(tds, NULL, 4);
+		/* Used to ignore next 4 bytes, now we'll actually parse (some of)
+			the data in them. (mlilback, 11/7/01) */
+		tds_get_n(tds, ci_flags, 4);
+		curcol->column_nullable = ci_flags[3] & 0x01;
+		curcol->column_writeable = (ci_flags[3] & 0x08) > 0;
+		curcol->column_identity = (ci_flags[3] & 0x10) > 0;
+		/* on with our regularly scheduled code (mlilback, 11/7/01) */
 		curcol->column_type = tds_get_byte(tds);
-		if (is_blob_type(curcol->column_type)) {
-			curcol->column_size = tds_get_int(tds);
-			tabnamesize = tds_get_smallint(tds);
-			tds_get_n(tds,NULL,tabnamesize);
-			bytes_read += 5+4+2+tabnamesize;
-		} else if (!is_fixed_type(curcol->column_type)) {
-			curcol->column_size = tds_get_byte(tds);
-			bytes_read += 5+1;
-		} else { 
-			curcol->column_size = get_size_by_type(curcol->column_type);
-			bytes_read += 5+0;
+
+		curcol->column_varint_size  = tds_get_varint_size(curcol->column_type);
+        tdsdump_log(TDS_DBG_INFO1, "%L processing result. type = %d, varint_size %d\n", 
+                    curcol->column_type, curcol->column_varint_size);
+
+		switch(curcol->column_varint_size) {
+			case 4: 
+				curcol->column_size = tds_get_int(tds);
+				/* junk the table name -- for now */
+			    tabnamesize = tds_get_smallint(tds);
+			    tds_get_n(tds,NULL,tabnamesize);
+			    bytes_read += 5+4+2+tabnamesize;
+				break;
+			case 1: 
+				curcol->column_size = tds_get_byte(tds);
+			    bytes_read += 5+1;
+				break;
+			case 0: 
+				curcol->column_size = get_size_by_type(curcol->column_type);
+			    bytes_read += 5+0;
+				break;
 		}
+
 		if (is_blob_type(curcol->column_type)) {
 			curcol->column_offset = info->row_size;
 		} else {
@@ -458,7 +540,7 @@ int remainder;
 	/* get the rest of the bytes */
 	rest = hdrsize - bytes_read;
 	if (rest > 0) {
-		fprintf(stderr,"NOTE:tds_process_col_info: draining %d bytes\n", rest);
+		tdsdump_log(TDS_DBG_INFO1,"NOTE:tds_process_col_info: draining %d bytes\n", rest);
 		tds_get_n(tds, NULL, rest);
 	}
 
@@ -562,8 +644,11 @@ static int tds7_process_result(TDSSOCKET *tds)
 {
 int col, num_cols;
 int colnamelen;
+TDS_SMALLINT tabnamelen;
 TDSCOLINFO *curcol;
 TDSRESULTINFO *info;
+TDS_SMALLINT collate_type;
+int remainder;
 
 	tds_free_all_results(tds);
 
@@ -580,44 +665,58 @@ TDSRESULTINFO *info;
 	for (col=0;col<num_cols;col++) {
 
 		curcol = info->columns[col];
-		tds_get_smallint(tds); /* ?  00 */
-		tds_get_smallint(tds); /* ?  01 */
+
+		/*  User defined data type of the column */
+		curcol->column_usertype = tds_get_smallint(tds);  
+
+        curcol->column_flags = tds_get_smallint(tds);     /*  Flags */
+
+        curcol->column_nullable  =  curcol->column_flags & 0x01;
+        curcol->column_writeable = (curcol->column_flags & 0x08) > 0;
+		curcol->column_identity  = (curcol->column_flags & 0x10) > 0;
+
 		curcol->column_type = tds_get_byte(tds); 
 		
-		/* large types (2 byte size field) are the type + 128
-		** except for nvarchar which is a varchar under 4.2 and
-		** doesn't have its own value 
-		*/
-		if (is_large_type(curcol->column_type)) {
-			curcol->column_xtype = curcol->column_type;
-			curcol->column_type -= 128;
-			if (curcol->column_xtype==XSYBNVARCHAR)
-				curcol->column_type=SYBVARCHAR;
-			if (curcol->column_xtype==XSYBNCHAR)
-				curcol->column_type=SYBCHAR;
+        curcol->column_type_save = curcol->column_type;
+		collate_type = is_collate_type(curcol->column_type);
+		curcol->column_varint_size  = tds_get_varint_size(curcol->column_type);
+
+		switch(curcol->column_varint_size) {
+			case 4: 
+				curcol->column_size = tds_get_int(tds);
+				break;
+			case 2: 
+				curcol->column_size = tds_get_smallint(tds);
+				break;
+			case 1: 
+				curcol->column_size = tds_get_byte(tds);
+				break;
+			case 0: 
+				curcol->column_size = get_size_by_type(curcol->column_type);
+				break;
 		}
 
-		/* text and image have 4 byte size field */
-		if (is_blob_type(curcol->column_type)) { 
-			curcol->column_size = tds_get_int(tds);
-			tds_get_string(tds,NULL,tds_get_smallint(tds));
-		/* fixed types have no size field */
-		} else if (is_fixed_type(curcol->column_type)) { 
-			curcol->column_size = 
-				get_size_by_type(curcol->column_type);
-		/* large char types have 2 byte field */
-		} else if (is_large_type(curcol->column_xtype)) {
-			curcol->column_size = tds_get_smallint(tds);
-		/* all others have 1 byte size */
-		} else { 
-			curcol->column_size = tds_get_byte(tds);
+		curcol->column_unicodedata = 0;
 
-			/* numeric and decimal have extra info */
-			if (is_numeric_type(curcol->column_type)) {
-				curcol->column_prec = tds_get_byte(tds); /* precision */
-				curcol->column_scale = tds_get_byte(tds); /* scale */
-			}
+		if (is_unicode(curcol->column_type))
+			curcol->column_unicodedata = 1;
 
+		curcol->column_type = tds_get_cardinal_type(curcol->column_type);
+
+		/* numeric and decimal have extra info */
+		if (is_numeric_type(curcol->column_type)) {
+			curcol->column_prec = tds_get_byte(tds); /* precision */
+			curcol->column_scale = tds_get_byte(tds); /* scale */
+		}
+
+		if (IS_TDS80(tds)) {
+			if (collate_type) 
+				tds_get_n(tds, curcol->collation, 5);
+		}
+
+		if (is_blob_type(curcol->column_type)) {
+			tabnamelen = tds_get_smallint(tds);
+			tds_get_string(tds, NULL, tabnamelen);
 		}
 
 		/* under 7.0 lengths are number of characters not 
@@ -636,6 +735,11 @@ TDSRESULTINFO *info;
 		if (is_numeric_type(curcol->column_type)) {
                        info->row_size += sizeof(TDS_NUMERIC) + 1;
 		}
+		
+		/* actually this 4 should be a machine dependent #define */
+		remainder = info->row_size % 4;
+		if (remainder) info->row_size += (4 - remainder);
+		
 	}
 
 	/* all done now allocate a row for tds_process_row to use */
@@ -681,21 +785,31 @@ int remainder;
 		tds_get_n(tds,curcol->column_name, colnamelen);
 		curcol->column_name[colnamelen]='\0';
 
-		tds_get_byte(tds); /* unknown */
+        curcol->column_flags = tds_get_byte(tds);     /*  Flags */
+        curcol->column_nullable  = (curcol->column_flags & 0x20) > 1;
+
 		curcol->column_usertype = tds_get_smallint(tds);
 		tds_get_smallint(tds);  /* another unknown */
 		curcol->column_type = tds_get_byte(tds);
 
-		if (is_blob_type(curcol->column_type)) {
-			curcol->column_size = tds_get_int(tds);
-			/* junk the table name -- for now */
-			tds_get_n(tds,NULL,tds_get_smallint(tds));
-		} else if (!is_fixed_type(curcol->column_type)) {
-			curcol->column_size = tds_get_byte(tds);
-		} else { 
-			curcol->column_size = get_size_by_type(curcol->column_type);
-		}
+		curcol->column_varint_size  = tds_get_varint_size(curcol->column_type);
 
+        tdsdump_log(TDS_DBG_INFO1, "%L processing result. type = %d, varint_size %d\n", curcol->column_type, curcol->column_varint_size);
+		switch(curcol->column_varint_size) {
+			case 4: 
+				curcol->column_size = tds_get_int(tds);
+				/* junk the table name -- for now */
+				tds_get_n(tds,NULL,tds_get_smallint(tds));
+				break;
+			case 1: 
+				curcol->column_size = tds_get_byte(tds);
+				break;
+			case 0: 
+				curcol->column_size = get_size_by_type(curcol->column_type);
+				break;
+			/* FIXME can varint_size be 2 ?? */
+		}
+        tdsdump_log(TDS_DBG_INFO1, "%L processing result. column_size %d\n", curcol->column_size);
 
 		/* numeric and decimal have extra info */
 		if (is_numeric_type(curcol->column_type)) {
@@ -759,40 +873,48 @@ unsigned char *dest;
 */
 static int tds_process_row(TDSSOCKET *tds)
 {
-int colsize, i, j, sign, num_pos;
-unsigned char b_val;
+int colsize, i;
 TDSCOLINFO *curcol;
 TDSRESULTINFO *info;
 unsigned char *dest;
 TDS_NUMERIC *num;
 TDS_VARBINARY *varbin;
-unsigned char temp_buf[8];
 int len;
 
 	info = tds->res_info;
+	if (!info)
+		return TDS_FAIL;
+
 	info->row_count++;
-	for (i=0;i<info->num_cols;i++)
-	{
+	for (i=0;i<info->num_cols;i++) {
 		curcol = info->columns[i];
-		if (is_blob_type(curcol->column_type)) {
-			/* length is always 16, if not we are in trouble */
-			len = tds_get_byte(tds);
-			if (len == 16) { /*  Jeff's hack */
-				tds_get_n(tds,curcol->column_textptr,16);
-				tds_get_n(tds,curcol->column_timestamp,8);
-				colsize = tds_get_int(tds);
-			} else {
-				colsize = 0;
-			}
-		} else if (is_large_type(curcol->column_xtype)) {
-			colsize = tds_get_smallint(tds);
-			if (colsize==-1) colsize=0;
-		} else if (!is_fixed_type(curcol->column_type)) {
-			colsize = tds_get_byte(tds);
-		} else {
-			colsize = get_size_by_type(curcol->column_type);
+        tdsdump_log(TDS_DBG_INFO1, "%L processing row.  column is %d varint size = %d\n", i, curcol->column_varint_size);
+		switch (curcol->column_varint_size) {
+			case 4: /* Its a BLOB... */
+				len = tds_get_byte(tds);
+				if (len == 16) { /*  Jeff's hack */
+					tds_get_n(tds,curcol->column_textptr,16);
+					tds_get_n(tds,curcol->column_timestamp,8);
+					colsize = tds_get_int(tds);
+				} else {
+					colsize = 0;
+				}
+				break;
+			case 2: 
+				/* FIXME add support for empty no-NULL string*/
+				colsize = tds_get_smallint(tds);
+				if (colsize == -1)
+					colsize=0;
+				break;
+			case 1: 
+				colsize = tds_get_byte(tds);
+				break;
+			case 0: 
+				colsize = get_size_by_type(curcol->column_type);
+				break;
 		}
 
+        tdsdump_log(TDS_DBG_INFO1, "%L processing row.  column size is %d \n", colsize);
 		/* set NULL flag in the row buffer */
 		if (colsize==0) {
 			tds_set_null(info->current_row, i);
@@ -800,6 +922,7 @@ int len;
 			tds_clr_null(info->current_row, i);
 
 			if (is_numeric_type(curcol->column_type)) {
+
 				/* 
 				** handling NUMERIC datatypes: 
 				** since these can be passed around independent
@@ -809,54 +932,18 @@ int len;
 				** it uses a few more bytes
 				*/
 				num = (TDS_NUMERIC *) &(info->current_row[curcol->column_offset]);
+                memset(num, '\0', sizeof(TDS_NUMERIC));
 				num->precision = curcol->column_prec;
 				num->scale = curcol->column_scale;
 
-	/*
 				tds_get_n(tds,num->array,colsize);
-				info->current_row[curcol->column_offset + sizeof(TDS_NUMERIC)]='\0';
-	*/
-				/*
-				** FIX: 02-Jan-2000 by Scott C. Gray (SCG)
-				** BUG: TDS70 Numeric Support
-				**
-				** I can think of _no_ reason why M$ decided to do this, but
-				** a TDS70 numeric seems to have the following properties over
-				** Sybase's.
-				**
-				**    1.  The sign bit is flipped.  1 = Positive, 0 = Negative
-				**    2.  The byte order is exactly opposite.  That is, in a
-				**        NUMERIC(10,0), the value '256' looks like:
-				**
-				**                    sign
-				**        sybase    = 000   000  000  000  001  000
-				**        microsoft = 001   000  001  000  000
-				**
-				** Also note that TDS 7.0 has two behaviors depending on which
-				** patch level you are running.  On SP1, the size of the numeric
-				** is always reports as 17 (the maximum number of bytes).  It
-				** seems that on SP2, M$ wised up and started packing the packet
-				** with only the required length to store the precision. Thus
-				** a NUMERIC(10,0) would look like:
-				**
-				**       SP1 = 17 bytes
-				**       SP2 = 5 bytes
-				*/
-				if (IS_TDS70(tds)) {
-					sign = tds_get_byte(tds);
-					num->array[0] = (sign == 0) ? 1 : 0;
-			
-					num_pos = g__numeric_bytes_per_prec[curcol->column_prec]-1;
-					for (j = 0; j < colsize - 1; j++)
-					{
-						b_val = tds_get_byte(tds);
-
-						if ((num_pos - j) > 0)
-							num->array[num_pos - j] = b_val;
-					}
-				} else {
-					tds_get_n(tds, num->array, colsize);
-				}
+				/* corrected colsize for cur_row_size */
+				colsize = sizeof(TDS_NUMERIC);
+		        if (IS_TDS70(tds) || IS_TDS80(tds)) 
+                {
+				   tdsdump_log(TDS_DBG_INFO1, "%L swapping numeric data...\n");
+                   tds_swap_datatype(tds_get_conversion_type(curcol->column_type, colsize), (unsigned char *)num );
+                }
 
 			} else if (curcol->column_type == SYBVARBINARY) {
 				varbin = (TDS_VARBINARY *) &(info->current_row[curcol->column_offset]);
@@ -871,21 +958,31 @@ int len;
 
 				tds_get_n(tds,varbin->array,colsize);
 			} else if (is_blob_type(curcol->column_type)) {
-				curcol->column_textvalue = realloc(curcol->column_textvalue,colsize);
+				if (curcol->column_unicodedata) colsize /= 2;
+				curcol->column_textvalue = realloc(curcol->column_textvalue,colsize+1); /* FIXME +1 needed by tds_get_string */
 				curcol->column_textsize = colsize;
-				tds_get_n(tds,curcol->column_textvalue,colsize);
+				if (curcol->column_unicodedata) {
+					tds_get_string(tds,curcol->column_textvalue,colsize);
+				} else {
+					tds_get_n(tds,curcol->column_textvalue,colsize);
+				}
 			} else {
 				dest = &(info->current_row[curcol->column_offset]);
-				if (is_unicode(curcol->column_xtype)) {
+				if (curcol->column_unicodedata) {
 					tds_get_string(tds,dest,colsize/2);
+					colsize /= 2;
 				} else {
 					tds_get_n(tds,dest,colsize);
 				}
+				/* FIXME correct for unicode ? */
 				dest[colsize]='\0';
+				if (curcol->column_type == SYBDATETIME4) {
+					tdsdump_log(TDS_DBG_INFO1, "%L datetime4 %d %d %d %d\n", dest[0], dest[1], dest[2], dest[3]);
+				}
 			}
-			if (curcol->column_type == SYBDATETIME4) {
-				tdsdump_log(TDS_DBG_INFO1, "%L datetime4 %d %d %d %d\n", dest[0], dest[1], dest[2], dest[3]);
-			}
+
+			/* Value used to properly know value in dbdatlen. (mlilback, 11/7/01) */
+			curcol->cur_row_size = colsize;
 
 #ifdef WORDS_BIGENDIAN
 			/* MS SQL Server 7.0 has broken date types from big endian 
@@ -903,17 +1000,30 @@ int len;
 				curcol->column_type == SYBDATETIMN ||
 				curcol->column_type == SYBMONEY ||
 				curcol->column_type == SYBMONEY4 ||
-				curcol->column_type == SYBMONEYN)) {
+				(curcol->column_type == SYBMONEYN && curcol->column_size > 4))) 
+				/* above line changed -- don't want this for 4 byte SYBMONEYN 
+					values (mlilback, 11/7/01) */
+			{
+				unsigned char temp_buf[8];
+
 				memcpy(temp_buf,dest,colsize/2);
 				memcpy(dest,&dest[colsize/2],colsize/2);
 				memcpy(&dest[colsize/2],temp_buf,colsize/2);
 			}
+            if (tds->emul_little_endian && !is_numeric_type(curcol->column_type)) {
+                tdsdump_log(TDS_DBG_INFO1, "%L swapping coltype %d\n",
+                            tds_get_conversion_type(curcol->column_type,colsize));
+                tds_swap_datatype(tds_get_conversion_type(curcol->column_type, colsize),
+                                  &(info->current_row[curcol->column_offset])
+                                 );
+            }
 #endif
 		}
 	}
 	return TDS_SUCCEED;
 }
 
+
 /*
 ** tds_process_end() processes any of the DONE, DONEPROC, or DONEINPROC
 ** tokens.
@@ -950,96 +1060,35 @@ int tmp = tds_get_smallint(tds);
 ** tds_client_msg() sends a message to the client application from the CLI or
 ** TDS layer. A client message is one that is generated from with the library
 ** and not from the server.  The message is sent to the CLI (the 
-** g_tds_err_handler) so that it may forward it to the client application or
+** err_handler) so that it may forward it to the client application or
 ** discard it if no msg handler has been by the application. tds->parent
 ** contains a void pointer to the parent of the tds socket. This can be cast
 ** back into DBPROCESS or CS_CONNECTION by the CLI and used to determine the
 ** proper recipient function for this message.
 */
-int tds_client_msg(TDSSOCKET *tds, int msgnum, int level, int state, int line, char *message)
+int tds_client_msg(TDSCONTEXT *tds_ctx, TDSSOCKET *tds, int msgnum, int level, int state, int line, char *message)
 {
 int ret;
-        if(tds->parent) {
-		tds->msg_info->msg_number=msgnum;
-        	tds->msg_info->msg_level=level; /* severity? */
-        	tds->msg_info->msg_state=state;
-        	tds->msg_info->server=strdup("OpenClient");
-        	tds->msg_info->line_number=line;
-        	tds->msg_info->message=strdup(message);
-        	ret = g_tds_err_handler(tds->parent);
+TDSMSGINFO msg_info;
+
+        if(tds_ctx->err_handler) {
+			memset(&msg_info, 0, sizeof(TDSMSGINFO));
+			msg_info.msg_number=msgnum;
+        	msg_info.msg_level=level; /* severity? */
+        	msg_info.msg_state=state;
+        	msg_info.server=strdup("OpenClient");
+        	msg_info.line_number=line;
+        	msg_info.message=strdup(message);
+        	ret = tds_ctx->err_handler(tds_ctx, tds, &msg_info);
 		/* message handler returned FAIL/CS_FAIL
 		** mark socket as dead */
-		if (ret) {
+		if (ret && tds) {
 			tds->state=TDS_DEAD;
 		}	
 	}
 	return 0;
 }
 
-/* 
-** tds_submit_query() sends a language string to the database server for
-** processing.  TDS 4.2 is a plain text message with a packet type of 0x01,
-** TDS 7.0 is a unicode string with packet type 0x01, and TDS 5.0 uses a 
-** TDS_LANG_TOKEN to encapsulate the query and a packet type of 0x0f.
-*/
-int tds_submit_query(TDSSOCKET *tds, char *query)
-{
-unsigned char *buf;
-int	bufsize;
-TDS_INT bufsize2;
-
-	if (!query) return TDS_FAIL;
-
-	/* Jeff's hack to handle long query timeouts */
-	tds->queryStarttime = time(NULL); 
-
-	if (tds->state==TDS_PENDING) {
-		/* FIX ME -- get real message number et al. 
-		** if memory serves the servername is 
-		** OpenClient for locally generated messages,
-		** but this needs to be verified too.
-		*/
-		tds_client_msg(tds,10000,7,0,1,
-        "Attempt to initiate a new SQL Server operation with results pending.");
-		return TDS_FAIL;
-	}
-
-	tds_free_all_results(tds);
-
-	tds->rows_affected = 0;
-	tds->state = TDS_QUERYING;
-	if (IS_TDS50(tds)) {
-		bufsize = strlen(query)+6;
-		buf = (unsigned char *) malloc(bufsize);
-		memset(buf,'\0',bufsize);
-		buf[0]=TDS_LANG_TOKEN; 
-
-		bufsize2 = strlen(query) + 1;
-		memcpy(buf+1, (void *)&bufsize2, 4);
-
-		memcpy(&buf[6],query,strlen(query));
-		tds->out_flag=0x0F;
-	} else if (IS_TDS70(tds)) {
-		bufsize = strlen(query)*2;
-		buf = (unsigned char *) malloc(bufsize);
-		memset(buf,'\0',bufsize);
-		tds7_ascii2unicode(query, buf, bufsize);
-		tds->out_flag=0x01;
-	} else { /* 4.2 */
-		bufsize = strlen(query);
-		buf = (unsigned char *) malloc(bufsize);
-		memset(buf,'\0',bufsize);
-		memcpy(&buf[0],query,strlen(query));
-		tds->out_flag=0x01;
-	}
-	tds_put_n(tds, buf, bufsize);
-	tds_flush_packet(tds);
-	
-	free(buf);
-
-	return TDS_SUCCEED;
-}
-
 /*
 ** tds_process_env_chg() 
 ** when ever certain things change on the server, such as database, character
@@ -1047,7 +1096,7 @@ TDS_INT bufsize2;
 ** There is no action taken currently, but certain functions at the CLI level
 ** that return the name of the current database will need to use this.
 */
-extern int tds_process_env_chg(TDSSOCKET *tds)
+int tds_process_env_chg(TDSSOCKET *tds)
 {
 int size, type;
 char *oldval, *newval;
@@ -1063,6 +1112,14 @@ TDSENVINFO *env = tds->env;
 
 	type = tds_get_byte(tds);
 
+	if (type==0x07) {
+		size = tds_get_byte(tds);
+		if (size) tds_get_n(tds, NULL, size);
+		size = tds_get_byte(tds);
+		if (size) tds_get_n(tds, NULL, size);
+		return TDS_SUCCEED;
+	}
+
 	/* fetch the new value */
 	size = tds_get_byte(tds);
 	newval = (char *) malloc((size+1)*2);
@@ -1137,6 +1194,9 @@ unsigned char *dest;
 ** tds_process_msg() is called for MSG, ERR, or EED tokens and is responsible
 ** for calling the CLI's message handling routine
 ** returns TDS_SUCCEED if informational, TDS_ERROR if error.
+**
+** Note: the called function is responsible for calling tds_reset_msg on the 
+** passed structure.
 */
 static int tds_process_msg(TDSSOCKET *tds,int marker)
 {
@@ -1146,11 +1206,15 @@ int len_msg;
 int len_svr;
 int len_sqlstate;
 
+	/* make sure message has been freed */
+	tds_free_msg(tds->msg_info);
+
 	/* packet length */
 	len = tds_get_smallint(tds);
 
 	/* message number */
-	tds->msg_info->msg_number = tds_get_int(tds);
+	rc = tds_get_int(tds);
+	tds->msg_info->msg_number = rc;
 
 	/* msg state */
 	tds->msg_info->msg_state = tds_get_byte(tds);
@@ -1179,7 +1243,7 @@ int len_sqlstate;
 	} else if (marker==TDS_ERR_TOKEN) {
 		tds->msg_info->priv_msg_type = 1;
 	} else {
-		fprintf(stderr,"tds_process_msg() called with unknown marker!\n");
+		tdsdump_log(TDS_DBG_ERROR,"tds_process_msg() called with unknown marker!\n");
 		return TDS_FAIL;
 	}
 
@@ -1201,7 +1265,7 @@ int len_sqlstate;
 		tds_unget_byte(tds);
 		tds->msg_info->proc_name=tds_msg_get_proc_name(tds);
 	} else {
-		tds->msg_info->proc_name=NULL;
+		tds->msg_info->proc_name=strdup("");
 	}
 
 	/* line number in the sql statement where the problem occured */
@@ -1217,11 +1281,12 @@ int len_sqlstate;
 	** the "parent" structure.
 	*/
 
-	if(tds->parent) {
-		if (tds->msg_info->priv_msg_type) 
-			g_tds_err_handler(tds->parent);
+	if(tds->msg_info->priv_msg_type
+	                   ? tds->tds_ctx->err_handler : tds->tds_ctx->msg_handler) {
+		if (tds->msg_info->priv_msg_type)
+			tds->tds_ctx->err_handler(tds->tds_ctx, tds, tds->msg_info);
 		else
-			g_tds_msg_handler(tds->parent);
+			tds->tds_ctx->msg_handler(tds->tds_ctx, tds, tds->msg_info);
 	} else {
 		if(tds->msg_info->msg_number)
 			tdsdump_log(TDS_DBG_WARN,
@@ -1244,47 +1309,36 @@ char *proc_name;
 
 	len_proc = tds_get_byte(tds);
 
-	if (len_proc > 0) {
-              /* its the length of a stored procedure name */
-              proc_name = (char*)malloc(len_proc+1);
-              tds_get_string(tds, proc_name, len_proc);
-              proc_name[len_proc] = '\0';
-	} else {
-
-              /* set the procname to null since there isn't one in the stream */
-              proc_name = (char*)NULL;
+	if (len_proc < 0) {
+		len_proc = 0;
 	}
+        proc_name = (char*)malloc(len_proc+1);
+	if (len_proc > 0) {
+              tds_get_string(tds, proc_name, len_proc);
+	}
+        proc_name[len_proc] = '\0';
 
 	return proc_name;
 
 }
 
-int tds_reset_msg_info(TDSSOCKET *tds)
+int tds_reset_msg_info(TDSMSGINFO *msg_info)
 {
-      tds->msg_info->priv_msg_type = 0;
-      tds->msg_info->msg_number = 0;
-      tds->msg_info->msg_state = 0;
-      tds->msg_info->msg_level = 0;
-      tds->msg_info->line_number = 0;
+	if (!msg_info) 
+		return 0;
 
-      if( tds->msg_info->message)
-              TDS_ZERO_FREE(tds->msg_info->message);
+	msg_info->priv_msg_type = 0;
+	msg_info->msg_number = 0;
+	msg_info->msg_state = 0;
+	msg_info->msg_level = 0;
+	msg_info->line_number = 0;
 
-      if(tds->msg_info->server)
-              TDS_ZERO_FREE(tds->msg_info->server);
-      if(tds->msg_info->proc_name)
-              TDS_ZERO_FREE(tds->msg_info->proc_name);
-	return 0;
-}
-
-/*
-** tds_send_cancel() sends an empty packet (8 byte header only)
-** tds_process_cancel should be called directly after this.
-*/
-int tds_send_cancel(TDSSOCKET *tds)
-{
-	tds->out_flag=0x06;
-	tds_flush_packet(tds);
+	if( msg_info->message)
+		TDS_ZERO_FREE(msg_info->message);
+	if(msg_info->server)
+		TDS_ZERO_FREE(msg_info->server);
+	if(msg_info->proc_name)
+		TDS_ZERO_FREE(msg_info->proc_name);
 
 	return 0;
 }
@@ -1302,6 +1356,8 @@ int marker, cancelled=0;
 		marker=tds_get_byte(tds);
 		if (marker==TDS_DONE_TOKEN) {
 			tds_process_end(tds, marker, NULL, &cancelled);
+		} else if (marker==0) {
+			cancelled = 1;
 		} else {
 			tds_process_default_tokens(tds,marker);
 		}
@@ -1398,7 +1454,7 @@ int bytenum = column  / 8;
 int bit = column % 8;
 unsigned char mask = 1 << bit;
 
-	tdsdump_log(TDS_DBG_INFO1,"%L setting column %d to NULL\n", column);
+	tdsdump_log(TDS_DBG_INFO1,"%L setting column %d NULL bit\n", column);
 	current_row[bytenum] |= mask;
 }
 /*
@@ -1410,7 +1466,7 @@ int bytenum = column  / 8;
 int bit = column % 8;
 unsigned char mask = ~(1 << bit);
 
-	tdsdump_log(TDS_DBG_INFO1, "%L setting column %d to NULL\n", column);
+	tdsdump_log(TDS_DBG_INFO1, "%L clearing column %d NULL bit\n", column);
 	current_row[bytenum] &= mask;
 }
 /*
@@ -1425,6 +1481,90 @@ unsigned char mask = 1 << bit;
 	return (current_row[bytenum] & mask) ? 1 : 0;
 }
 
+int tds_lookup_dynamic(TDSSOCKET *tds, char *id)
+{
+int i;
+
+	for (i=0;i<tds->num_dyns;i++) {
+		if (!strcmp(tds->dyns[i]->id, id)) {
+			return i;
+		}
+	}
+	return -1;
+}
+/*
+** tds_process_dynamic()
+** finds the element of the dyns array for the id
+*/
+static int tds_process_dynamic(TDSSOCKET *tds)
+{
+int token_sz;
+char subtoken[2];
+int id_len;
+char id[TDS_MAX_DYNID_LEN+1];
+int drain = 0;
+
+	token_sz = tds_get_smallint(tds);
+	subtoken[0] = tds_get_byte(tds);
+	subtoken[1] = tds_get_byte(tds);
+	if (subtoken[0]!=0x20 || subtoken[1]!=0x00) {
+		tdsdump_log(TDS_DBG_ERROR,"Unrecognized TDS5_DYN subtoken %x,%x\n",
+		        subtoken[0], subtoken[1]);
+		tds_get_n(tds, NULL, token_sz-2);
+		return -1;
+	}
+	id_len = tds_get_byte(tds);
+	if (id_len > TDS_MAX_DYNID_LEN) {
+		drain = id_len - TDS_MAX_DYNID_LEN;
+		id_len = TDS_MAX_DYNID_LEN;
+	}
+	tds_get_string(tds, id, id_len);
+	id[id_len]='\0';
+	if (drain) { 
+		tds_get_string(tds, NULL, drain);
+	}
+	return tds_lookup_dynamic(tds,id);
+}
+
+static void tds_process_dyn_result(TDSSOCKET *tds)
+{
+int hdrsize;
+int col, num_cols;
+TDSCOLINFO *curcol;
+TDSRESULTINFO *info;
+TDSDYNAMIC *dyn;
+
+	hdrsize = tds_get_smallint(tds);
+	num_cols = tds_get_smallint(tds);
+
+	if (tds->cur_dyn_elem) {
+		dyn = tds->dyns[tds->cur_dyn_elem];
+		tds_free_results(dyn->res_info);
+		/* read number of columns and allocate the columns structure */
+		dyn->res_info = tds_alloc_results(num_cols);
+		info = dyn->res_info;
+	} else {
+		tds_free_results(tds->res_info);
+		tds->res_info = tds_alloc_results(num_cols);
+		info = tds->res_info;
+	}
+
+	for (col=0;col<info->num_cols;col++) {
+		curcol=info->columns[col];
+		tds_get_n(tds,NULL,6);
+		/* column type */
+		curcol->column_type = tds_get_byte(tds);
+		/* column size */
+		if (!is_fixed_type(curcol->column_type)) {
+			curcol->column_size = tds_get_byte(tds);
+		} else { 
+			curcol->column_size = get_size_by_type(curcol->column_type);
+		}
+		tds_get_byte(tds);
+		/* fprintf(stderr,"elem %d coltype %d size %d\n",tds->cur_dyn_elem, curcol->column_type, curcol->column_size); */
+	}
+}
+
 /* 
 ** tds_is_fixed_token()
 ** some tokens are fixed length while others are variable.  This function is 
@@ -1462,3 +1602,100 @@ int tds_get_token_size(int marker)
 			return 0;
 	}
 }
+void tds_swap_datatype(int coltype, unsigned char *buf)
+{
+TDS_NUMERIC *num;
+
+	switch(coltype) {
+		case SYBINT2:
+			tds_swap_bytes(buf,2); break;
+		case SYBINT4:
+			tds_swap_bytes(buf,4); break;
+		case SYBINT8:
+			tds_swap_bytes(buf,8); break;
+		case SYBREAL:
+			tds_swap_bytes(buf,4); break;
+		case SYBFLT8:
+			tds_swap_bytes(buf,8); break;
+		case SYBMONEY4:
+			tds_swap_bytes(buf,4); break;
+		case SYBMONEY:
+			tds_swap_bytes(buf,4);
+			tds_swap_bytes(&buf[4],4); break;
+		case SYBDATETIME4:
+			tds_swap_bytes(buf,2);
+			tds_swap_bytes(&buf[2],2); break;
+		case SYBDATETIME:
+			tds_swap_bytes(buf,4);
+			tds_swap_bytes(&buf[4],4); break;
+		case SYBNUMERIC:
+		case SYBDECIMAL:
+			num = (TDS_NUMERIC *) buf;
+            /* swap the sign */
+            num->array[0] = (num->array[0] == 0) ? 1 : 0;
+            /* swap the data */
+            tds_swap_bytes(&(num->array[1]),
+                           g__numeric_bytes_per_prec[num->precision] - 1); break;
+
+        case SYBUNIQUE:
+			tds_swap_bytes(buf,4);
+			tds_swap_bytes(&buf[4],2); 
+			tds_swap_bytes(&buf[6],2); break;
+
+	}
+}
+/*
+** tds_get_varint_size() returns the size of a variable length integer
+** returned in a TDS 7.0 result string
+*/
+static int tds_get_varint_size(int datatype)
+{
+	switch(datatype) {
+		case SYBTEXT:
+		case SYBNTEXT:
+		case SYBIMAGE:
+		case SYBVARIANT:
+			return 4;
+		case SYBVOID:
+		case SYBINT1:
+		case SYBBIT:
+		case SYBINT2:
+		case SYBINT4:
+		case SYBDATETIME4:
+		case SYBREAL:
+		case SYBMONEY:
+		case SYBDATETIME:
+		case SYBFLT8:
+		case SYBMONEY4:
+		case SYBINT8:
+			return 0;
+		case XSYBNCHAR:
+		case XSYBNVARCHAR:
+		case XSYBCHAR:
+		case XSYBVARCHAR:
+		case XSYBBINARY:
+		case XSYBVARBINARY:
+			return 2;
+		default:
+			return 1;
+	}
+}
+
+static int tds_get_cardinal_type(int datatype)
+{
+	switch(datatype) {
+		case XSYBVARBINARY: 
+			return SYBVARBINARY;
+		case XSYBBINARY: 
+			return SYBBINARY;
+		case SYBNTEXT:
+			return SYBTEXT;
+		case XSYBNVARCHAR: 
+		case XSYBVARCHAR: 
+			return SYBVARCHAR;
+		case XSYBNCHAR: 
+		case XSYBCHAR: 
+			return SYBCHAR;
+	}
+	return datatype;
+}
