@@ -43,9 +43,12 @@
 #include <arpa/inet.h>
 #endif
 #include "tdsutil.h"
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
 
 
-static char  software_version[]   = "$Id: util.c,v 1.5 2001/11/10 02:39:01 brianb Exp $";
+static char  software_version[]   = "$Id: util.c,v 1.11 2002/08/09 02:55:49 brianb Exp $";
 static void *no_unused_var_warn[] = {software_version,
                                      no_unused_var_warn};
 
@@ -53,16 +56,31 @@ static void *no_unused_var_warn[] = {software_version,
 int g_debug_lvl = 99;
 int g_append_mode = 0;
 static char *g_dump_filename;
+static int   write_dump = 0;      /* is TDS stream debug log turned on? */
+static FILE *dumpfile   = NULL;   /* file pointer for dump log          */
 
-void tds_set_parent(TDSSOCKET* tds, void* the_parent)
+void 
+tds_set_parent(TDSSOCKET *tds, void *the_parent)
 {
 	if (tds)
       		tds->parent = the_parent;
 }
 
-void* tds_get_parent(TDSSOCKET* tds)
+void *
+tds_get_parent(TDSSOCKET *tds)
 {
-      return( tds->parent);
+      return(tds->parent);
+}
+void 
+tds_ctx_set_parent(TDSCONTEXT *ctx, void* the_parent)
+{
+	if (ctx)
+      		ctx->parent = the_parent;
+}
+void *
+tds_ctx_get_parent(TDSCONTEXT *ctx)
+{
+      return(ctx->parent);
 }
 
 int tds_swap_bytes(unsigned char *buf, int bytes)
@@ -79,12 +97,6 @@ int i;
 	return bytes;
 }
 
-static int   write_dump = 0;      /* is TDS stream debug log turned on? */
-static FILE *dumpfile   = NULL;   /* file pointer for dump log          */
-static pid_t pid;
-
-
-
 /* ============================== tdsdump_off() ==============================
  *
  * Def:  temporarily turn off logging.  Note- 
@@ -129,8 +141,6 @@ int tdsdump_open(const char *filename)
 {
 int   result;   /* really should be a boolean, not an int */
 
-   pid = getpid();
-
    if (filename == NULL || filename[0]=='\0') {
       filename = "tdsdump.out";
    }
@@ -187,14 +197,14 @@ int result;
  */
 void tdsdump_close()
 {
-   if (dumpfile!=NULL)
+   tdsdump_off();
+   if (dumpfile!=NULL && dumpfile != stdout && dumpfile != stderr)
    {
       fclose(dumpfile);
    }
    if (g_dump_filename) {
       free(g_dump_filename);
    }
-   tdsdump_off();
 } /* tdsdump_close()  */
 
 
@@ -231,6 +241,7 @@ void tdsdump_dump_buf(
          for(j=i; j<length && (j-i)<bytesPerLine; j++)
          {
             fprintf(dumpfile, "%02x ", data[j]);
+            if (j-i == bytesPerLine/2) fprintf(dumpfile, " ");
          }
          
          /*
@@ -248,6 +259,7 @@ void tdsdump_dump_buf(
          for(j=i; j<length && (j-i)<bytesPerLine; j++)
          {
             fprintf(dumpfile, "%c", (isprint(data[j])) ? data[j] : '.');
+            if (j-i == bytesPerLine/2) fprintf(dumpfile, " ");
          }
          fprintf(dumpfile, "|\n");
       }
@@ -292,7 +304,7 @@ void tdsdump_log(int debug_lvl, const char *fmt, ...)
       va_start(ap, fmt);
       
    	 if (g_append_mode) {
-          fprintf(dumpfile, "pid: %d:", pid);
+          fprintf(dumpfile, "pid: %d:", (int)getpid() );
       }
       for(ptr = fmt; *ptr != '\0'; ptr++)
       {
@@ -328,30 +340,9 @@ void tdsdump_log(int debug_lvl, const char *fmt, ...)
                }
                case 'L': /* current local time */
                {
-                  char        buf[1024];
-                  struct tm  *tm;
-                  time_t      t;
+                  char        buf[128];
 
-#ifdef __FreeBSD__
-                  struct timeval  tv;
-#endif
-
-#ifdef __FreeBSD__
-                  gettimeofday(&tv, NULL);
-                  t = tv.tv_sec;
-#else
-                  /* 
-                   * XXX Need to get a better time resolution for 
-                   * non-FreeBSD systems.
-                   */
-                  time(&t);
-#endif
-                  tm = localtime(&t);
-                  strftime(buf, sizeof(buf)-1, "%Y-%m-%d %H:%M:%S", tm);
-                  fputs(buf, dumpfile);
-#ifdef __FreeBSD__
-                  fprintf(dumpfile, ".%06lu", tv.tv_usec);
-#endif
+                  fputs(tds_timestamp_str(buf, 127), dumpfile);
                }
                default:
                {
@@ -390,7 +381,7 @@ int tds_msleep(long usec)         /* returns 0 if ok, else -1 */
 
     return(select(0, (fd_set *)NULL, (fd_set *)NULL, (fd_set *)NULL, &delay));
 #endif
-    };
+    }
 /* Jeff's hack ***NEW CODE END**** */
 
 
